@@ -1,28 +1,24 @@
 (* camlp5 pa_r.cmo pa_rp.cmo pa_extend.cmo q_MLast.cmo pr_dump.cmo *)
 (* pa_lisp.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 open Pcaml;
 open Versdep;
 
-type choice α β =
-  [ Left of α
-  | Right of β ]
-;
-
 (* Buffer *)
 
 module Buff =
   struct
     value buff = ref (string_create 80);
     value store len x = do {
-      if len >= String.length buff.val then
-        buff.val := buff.val ^ string_create (String.length buff.val)
+      if len >= string_length buff.val then
+        buff.val :=
+          string_cat buff.val (string_create (string_length buff.val))
       else ();
       string_set buff.val len x;
       succ len
     };
-    value get len = String.sub buff.val 0 len;
+    value get len = bytes_to_string (string_sub buff.val 0 len);
   end
 ;
 
@@ -221,6 +217,12 @@ value expr_id loc s =
   | _ → <:expr< $lid:s$ >> ]
 ;
 
+value is_lident s =
+  match s.[0] with
+  [ 'A'..'Z' → False
+  | _ → True ]
+;
+
 value patt_id loc s =
   match s.[0] with
   [ 'A'..'Z' → <:patt< $uid:s$ >>
@@ -232,10 +234,35 @@ value ctyp_id loc s =
   [ ''' →
       let s = String.sub s 1 (String.length s - 1) in
       <:ctyp< '$s$ >>
-  | 'A'..'Z' → <:ctyp< $uid:s$ >>
+  | 'A'..'Z' → failwith "must not call ctyp_id with UIDs"
   | _ → <:ctyp< $lid:s$ >> ]
 ;
 
+value split_at_dots loc s =
+  loop 0 0 where rec loop ibeg i =
+    if i = String.length s then
+      if i > ibeg then [ (String.sub s ibeg (i - ibeg)) ]
+      else Ploc.raise (Ploc.sub loc (i - 1) 1) (Stream.Error "ctyp expected")
+    else if s.[i] = '.' then
+      if i > ibeg then
+        let t1 = [ (String.sub s ibeg (i - ibeg)) ] in
+        let t2 = loop (i + 1) (i + 1) in
+        t1 @ t2
+      else Ploc.raise (Ploc.sub loc (i - 1) 1) (Stream.Error "ctyp expected")
+    else loop ibeg (i + 1)
+;
+
+value split_last l =
+  match List.rev l with [
+    [] -> failwith "split_last: empty list"
+  | [ h :: t ] -> (List.rev t, h)
+  ]
+;
+
+value capitalized s =
+  match s.[0] with [ 'A'..'Z' -> True | _ -> False ]
+;
+
 value strm_n = "strm__";
 value peek_fun loc = <:expr< Stream.peek >>;
 value junk_fun loc = <:expr< Stream.junk >>;
@@ -256,8 +283,7 @@ and str_item_se se =
       let mb = module_binding_se se in
       <:str_item< module $uid:i$ = $mb$ >>
   | Sexpr loc [Satom _ Alid "open"; Satom _ Auid s] →
-      let s = [s] in
-      <:str_item< open $s$ >>
+      <:str_item< open $uid:s$ >>
   | Sexpr loc [Satom _ Alid "type" :: sel] →
       let tdl = type_declaration_list_se sel in
       <:str_item< type $list:tdl$ >>
@@ -274,7 +300,7 @@ and str_item_se se =
       <:str_item< $exp:e$ >> ]
 and value_binding_se =
   fun
-  [ [se1; se2 :: sel] → [(ipatt_se se1, expr_se se2) :: value_binding_se sel]
+  [ [se1; se2 :: sel] → [(ipatt_se se1, expr_se se2, <:vala< [] >>) :: value_binding_se sel]
   | [] → []
   | [se :: _] → error se "value_binding" ]
 and module_binding_se se = module_expr_se se
@@ -326,7 +352,7 @@ and expr_se =
       [ [Sexpr _ sel1 :: sel2] →
           List.fold_right
             (fun se ek →
-               let (p, e) = let_binding_se se in
+               let (p, e, _) = let_binding_se se in
                <:expr< let $p$ = $e$ in $ek$ >>)
             sel1 (progn_se loc sel2)
       | [se :: _] → error se "let_binding"
@@ -403,7 +429,7 @@ and progn_se loc =
       <:expr< do { $list:el$ } >> ]
 and let_binding_se =
   fun
-  [ Sexpr loc [se1; se2] → (ipatt_se se1, expr_se se2)
+  [ Sexpr loc [se1; se2] → (ipatt_se se1, expr_se se2, <:vala< [] >>)
   | se → error se "let_binding" ]
 and match_case loc =
   fun
@@ -418,19 +444,9 @@ and label_expr_se loc =
 and expr_ident_se loc s =
   if s.[0] = '<' then <:expr< $lid:s$ >>
   else
-    loop 0 0 where rec loop ibeg i =
-      if i = String.length s then
-        if i > ibeg then expr_id loc (String.sub s ibeg (i - ibeg))
-        else
-          Ploc.raise (Ploc.sub loc (i - 1) 1) (Stream.Error "expr expected")
-      else if s.[i] = '.' then
-        if i > ibeg then
-          let e1 = expr_id loc (String.sub s ibeg (i - ibeg)) in
-          let e2 = loop (i + 1) (i + 1) in
-          <:expr< $e1$ . $e2$ >>
-        else
-          Ploc.raise (Ploc.sub loc (i - 1) 1) (Stream.Error "expr expected")
-      else loop ibeg (i + 1)
+    let l = Versdep.split_on_char '.' s in
+    Asttools.expr_of_string_list loc l
+
 and parser_cases_se loc =
   fun
   [ [] → <:expr< raise Stream.Failure >>
@@ -527,17 +543,18 @@ and patt_se =
   | Sexpr loc [] → <:patt< () >>
   | Squot loc typ txt → Pcaml.handle_patt_quotation loc (typ, txt) ]
 and patt_ident_se loc s =
-  loop 0 0 where rec loop ibeg i =
-    if i = String.length s then
-      if i > ibeg then patt_id loc (String.sub s ibeg (i - ibeg))
-      else Ploc.raise (Ploc.sub loc (i - 1) 1) (Stream.Error "patt expected")
-    else if s.[i] = '.' then
-      if i > ibeg then
-        let p1 = patt_id loc (String.sub s ibeg (i - ibeg)) in
-        let p2 = loop (i + 1) (i + 1) in
-        <:patt< $p1$ . $p2$ >>
-      else Ploc.raise (Ploc.sub loc (i - 1) 1) (Stream.Error "patt expected")
-    else loop ibeg (i + 1)
+  let sl = split_at_dots loc s in
+  let (hdl, lid) = split_last sl in do {
+  if not (List.for_all capitalized hdl) then
+    Ploc.raise loc (Stream.Error "patt expected, but components aren't capitalized")
+  else () ;
+  match hdl with [
+    [] -> patt_id loc lid
+  | [ h :: t ] ->
+    let me = List.fold_left (fun me uid -> <:extended_longident< $longid:me$ . $uid:uid$ >>)
+      <:extended_longident< $uid:h$ >> t in
+    <:patt< $longid:me$ . $lid:lid$ >>
+  ]}
 and ipatt_se se =
   match ipatt_opt_se se with
   [ Left p → p
@@ -565,9 +582,11 @@ and type_declaration_list_se =
       let empty = [] in
       let n = (loc1, <:vala< n1 >>) in
       let td =
-        {MLast.tdNam = <:vala< n >>; MLast.tdPrm = <:vala< tpl >>;
+        {MLast.tdIsDecl = <:vala< True >> ;
+         MLast.tdNam = <:vala< n >>; MLast.tdPrm = <:vala< tpl >>;
          MLast.tdPrv = <:vala< False >>; MLast.tdDef = ctyp_se se2;
-         MLast.tdCon = <:vala< empty >>}
+         MLast.tdCon = <:vala< empty >>;
+         MLast.tdAttributes = <:vala< [] >>}
       in
       [td :: type_declaration_list_se sel]
   | [] → []
@@ -576,7 +595,7 @@ and type_parameter_se =
   fun
   [ Satom _ Alid s when String.length s >= 2 && s.[0] = ''' →
       let s = String.sub s 1 (String.length s - 1) in
-      (<:vala< (Some s) >>, None)
+      (<:vala< (Some s) >>, (None, False))
   | se → error se "type_parameter" ]
 and ctyp_se =
   fun
@@ -592,21 +611,22 @@ and ctyp_se =
   | Satom loc (Alid | Auid) s → ctyp_ident_se loc s
   | se → error se "ctyp" ]
 and ctyp_ident_se loc s =
-  loop 0 0 where rec loop ibeg i =
-    if i = String.length s then
-      if i > ibeg then ctyp_id loc (String.sub s ibeg (i - ibeg))
-      else Ploc.raise (Ploc.sub loc (i - 1) 1) (Stream.Error "ctyp expected")
-    else if s.[i] = '.' then
-      if i > ibeg then
-        let t1 = ctyp_id loc (String.sub s ibeg (i - ibeg)) in
-        let t2 = loop (i + 1) (i + 1) in
-        <:ctyp< $t1$ . $t2$ >>
-      else Ploc.raise (Ploc.sub loc (i - 1) 1) (Stream.Error "ctyp expected")
-    else loop ibeg (i + 1)
+  let sl = split_at_dots loc s in
+  let (hdl, lid) = split_last sl in do {
+  if not (List.for_all capitalized hdl) then
+    Ploc.raise loc (Stream.Error "ctyp expected, but components aren't capitalized")
+  else () ;
+  match hdl with [
+    [] -> ctyp_id loc lid
+  | [ h :: t ] ->
+    let me = List.fold_left (fun me uid -> <:extended_longident< $longid:me$ . $uid:uid$ >>)
+      <:extended_longident< $uid:h$ >> t in
+    <:ctyp< $longid:me$ . $lid:lid$ >>
+  ]}
 and constructor_declaration_se =
   fun
   [ Sexpr loc [Satom _ Auid ci :: sel] →
-      (loc, <:vala< ci >>, <:vala< (List.map ctyp_se sel) >>, None)
+      (loc, <:vala< ci >>, <:vala< [] >>, <:vala< (List.map ctyp_se sel) >>, <:vala< None >>, <:vala< [] >>)
   | se → error se "constructor_declaration" ]
 ;
 
@@ -650,44 +670,46 @@ do {
   Grammar.Unsafe.clear_entry class_str_item
 };
 
-Pcaml.parse_interf.val := Grammar.Entry.parse interf;
-Pcaml.parse_implem.val := Grammar.Entry.parse implem;
+Pcaml.(set_ast_parse transduce_interf (Grammar.Entry.parse interf));
+Pcaml.(set_ast_parse transduce_implem (Grammar.Entry.parse implem));
+Pcaml.(set_ast_parse transduce_top_phrase (Grammar.Entry.parse top_phrase));
+Pcaml.(set_ast_parse transduce_use_file (Grammar.Entry.parse use_file));
 
 value sexpr = Grammar.Entry.create gram "sexpr";
 value atom = Grammar.Entry.create gram "atom";
 
 EXTEND
   implem:
-    [ [ st = LIST0 [ s = str_item -> (s, loc) ]; eoi_loc = [ EOI -> loc ] ->
+    [ [ st = LIST0 [ s = str_item → (s, loc) ]; eoi_loc = [ EOI → loc ] →
           (st, Some eoi_loc) ] ]
   ;
   top_phrase:
-    [ [ se = sexpr -> Some (top_phrase_se se)
-      | EOI -> None ] ]
+    [ [ se = sexpr → Some (top_phrase_se se)
+      | EOI → None ] ]
   ;
   use_file:
-    [ [ l = LIST0 sexpr; EOI -> (List.map top_phrase_se l, False) ] ]
+    [ [ l = LIST0 sexpr; EOI → (List.map top_phrase_se l, False) ] ]
   ;
   str_item:
-    [ [ se = sexpr -> str_item_se se
-      | e = expr -> <:str_item< $exp:e$ >> ] ]
+    [ [ se = sexpr → str_item_se se
+      | e = expr → <:str_item< $exp:e$ >> ] ]
   ;
   expr:
     [ "top"
-      [ se = sexpr -> expr_se se ] ]
+      [ se = sexpr → expr_se se ] ]
   ;
   patt:
-    [ [ se = sexpr -> patt_se se ] ]
+    [ [ se = sexpr → patt_se se ] ]
   ;
   sexpr:
-    [ [ "("; sl = LIST0 sexpr; ")" -> Sexpr loc sl
-      | a = atom -> Satom loc Alid a
-      | s = LIDENT -> Satom loc Alid s
-      | s = UIDENT -> Satom loc Auid s
-      | s = INT -> Satom loc Aint s
-      | s = CHAR -> Satom loc Achar s
-      | s = STRING -> Satom loc Astring s
-      | s = QUOT ->
+    [ [ "("; sl = LIST0 sexpr; ")" → Sexpr loc sl
+      | a = atom → Satom loc Alid a
+      | s = LIDENT → Satom loc Alid s
+      | s = UIDENT → Satom loc Auid s
+      | s = INT → Satom loc Aint s
+      | s = CHAR → Satom loc Achar s
+      | s = STRING → Satom loc Astring s
+      | s = QUOT →
           let i = String.index s ':' in
           let typ = String.sub s 0 i in
           let txt = String.sub s (i + 1) (String.length s - i - 1) in
@@ -696,10 +718,10 @@ EXTEND
           else Squot loc typ txt ] ]
   ;
   atom:
-    [ [ "_" -> "_"
-      | "," -> ","
-      | "=" -> "="
-      | ":" -> ":"
-      | "." -> "." ] ]
+    [ [ "_" → "_"
+      | "," → ","
+      | "=" → "="
+      | ":" → ":"
+      | "." → "." ] ]
   ;
 END;
