@@ -1749,8 +1749,10 @@ do {
   Grammar.Unsafe.clear_entry class_str_item
 };
 
-Pcaml.parse_interf.val := Grammar.Entry.parse interf;
-Pcaml.parse_implem.val := Grammar.Entry.parse implem;
+Pcaml.(set_ast_parse transduce_interf (Grammar.Entry.parse interf));
+Pcaml.(set_ast_parse transduce_implem (Grammar.Entry.parse implem));
+Pcaml.(set_ast_parse transduce_top_phrase (Grammar.Entry.parse top_phrase));
+Pcaml.(set_ast_parse transduce_use_file (Grammar.Entry.parse use_file));
 
 value sexpr = Grammar.Entry.create gram "sexpr";
 
