@@ -1,15 +1,26 @@
 (* camlp5r *)
 (* pcaml.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2019 *)
 
 #load "pa_macro.cmo";
 #load "pa_extend.cmo";
 
 open Printf;
 
-value version = "6.16";
+value version = "8.02.01";
 value syntax_name = ref "";
 
+value ocaml_version =
+  loop 0 where rec loop i =
+    if i = String.length Versdep.sys_ocaml_version then
+      Versdep.sys_ocaml_version
+    else
+      match Versdep.sys_ocaml_version.[i] with
+      | ' ' | '+' | '~' -> String.sub Versdep.sys_ocaml_version 0 i
+      | _ -> loop (i + 1)
+      end
+;
+
 value gram =
   Grammar.gcreate
     {Plexing.tok_func _ = failwith "no loaded parsing module";
@@ -18,8 +29,17 @@ value gram =
      Plexing.tok_comm = None}
 ;
 
+(*
+Camlp5 can be parsed with limited or full backtracking:
+Grammar.set_algorithm gram Grammar.Functional;
+Grammar.set_algorithm gram Grammar.Backtracking;
+or without any change in the code, by setting the environment
+variable CAMLP5PARAM to f or b.
+*)
+
 type status = option Ploc.t;
 
+value attribute_body = Grammar.Entry.create gram "attribute_body";
 value interf = Grammar.Entry.create gram "interf";
 value implem = Grammar.Entry.create gram "implem";
 value top_phrase = Grammar.Entry.create gram "top_phrase";
@@ -28,7 +48,11 @@ value signature = Grammar.Entry.create gram "signature";
 value structure = Grammar.Entry.create gram "structure";
 value sig_item = Grammar.Entry.create gram "sig_item";
 value str_item = Grammar.Entry.create gram "str_item";
+value functor_parameter = Grammar.Entry.create gram "functor_parameter";
 value module_type = Grammar.Entry.create gram "module_type";
+value longident = Grammar.Entry.create gram "longident";
+value longident_lident = Grammar.Entry.create gram "longident_lident";
+value extended_longident = Grammar.Entry.create gram "extended_longident";
 value module_expr = Grammar.Entry.create gram "module_expr";
 value expr = Grammar.Entry.create gram "expr";
 value patt = Grammar.Entry.create gram "patt";
@@ -36,6 +60,8 @@ value ipatt = Grammar.Entry.create gram "ipatt";
 value ctyp = Grammar.Entry.create gram "ctyp";
 value let_binding = Grammar.Entry.create gram "let_binding";
 value type_decl = Grammar.Entry.create gram "type_declaration";
+value type_extension = Grammar.Entry.create gram "type_extension";
+value extension_constructor = Grammar.Entry.create gram "extension_constructor";
 value match_case = Grammar.Entry.create gram "match_case";
 value constructor_declaration =
   Grammar.Entry.create gram "constructor_declaration";
@@ -48,9 +74,53 @@ value class_sig_item = Grammar.Entry.create gram "class_sig_item";
 value class_str_item = Grammar.Entry.create gram "class_str_item";
 value class_type = Grammar.Entry.create gram "class_type";
 value class_expr = Grammar.Entry.create gram "class_expr";
+value class_expr_simple = Grammar.Entry.create gram "class_expr_simple";
 
-value parse_interf = ref (Grammar.Entry.parse interf);
-value parse_implem = ref (Grammar.Entry.parse implem);
+value alg_attribute = Grammar.Entry.create gram "alg_attribute";
+value alg_attributes = Grammar.Entry.create gram "alg_attributes";
+value ext_attributes = Grammar.Entry.create gram "ext_attributes";
+
+type ast_transducer_t 'a = {
+    name : string
+  ; parse : ref (option (Stream.t char -> 'a))
+  ; transform : ref (option ('a -> 'a))
+  } ;
+
+value set_ast_parse att pf =
+  match att.parse.val with [
+      Some _ -> failwith (Printf.sprintf "Pcaml.set_ast_parse: transducer \"%s\" already has a parse(r)" att.name)
+    | None -> att.parse.val := Some pf
+    ]
+;
+
+value set_ast_transform att tf =
+  match att.transform.val with [
+      Some _ -> failwith (Printf.sprintf "Pcaml.set_ast_transform: transducer \"%s\" already has a tranform(er)" att.name)
+    | None -> att.transform.val := Some tf
+    ]
+;
+
+value transduce att x =
+  let parse = match att.parse.val with [
+        None -> failwith (Printf.sprintf "Pcaml.transduce: transducer \"%s\" has no configured parser" att.name)
+      | Some x -> x
+      ] in
+  let x = parse x in
+  match att.transform.val with [
+      None -> x
+    | Some f -> f x
+    ]
+;
+
+value transduce_interf = { name = "interf" ; parse = ref None ; transform = ref None } ;
+value transduce_implem = { name = "implem" ; parse = ref None ; transform = ref None } ;
+value transduce_top_phrase = { name = "top_phrase" ; parse = ref None ; transform = ref None } ;
+value transduce_use_file = { name = "use_file" ; parse = ref None ; transform = ref None } ;
+
+value parse_interf x = transduce transduce_interf x ;
+value parse_implem x = transduce transduce_implem x ;
+value parse_top_phrase x = transduce transduce_top_phrase x ;
+value parse_use_file x = transduce transduce_use_file x ;
 
 value rec skip_to_eol cs =
   match Stream.peek cs with
@@ -83,7 +153,7 @@ type err_ctx =
   | Expanding
   | ParsingResult of Ploc.t and string ]
 ;
-exception Qerror of string and err_ctx and exn;
+exception Qerror of string and string and err_ctx and exn;
 
 value quotation_location () =
   match quotation_loc.val with
@@ -114,7 +184,7 @@ value expand_quotation gloc expander shift name str = do {
      try
        try expander str with
        [ Ploc.Exc loc exc ->
-           let exc1 = Qerror name Expanding exc in
+           let exc1 = Qerror name str Expanding exc in
            let shift = Ploc.first_pos gloc + shift in
            let loc =
              let gloc_line_nb = Ploc.line_nb gloc in
@@ -131,7 +201,7 @@ value expand_quotation gloc expander shift name str = do {
            in
            raise (Ploc.Exc loc exc1)
        | exc ->
-           let exc1 = Qerror name Expanding exc in
+           let exc1 = Qerror name str Expanding exc in
            Ploc.raise gloc exc1 ]
     with
     [ exn -> do { restore (); raise exn } ]
@@ -143,15 +213,15 @@ value expand_quotation gloc expander shift name str = do {
 value parse_quotation_result entry loc shift name str =
   let cs = Stream.of_string str in
   try Grammar.Entry.parse entry cs with
-  [ Ploc.Exc iloc (Qerror _ Expanding exc) ->
+  [ Ploc.Exc iloc (Qerror _ _ Expanding exc) ->
       let ctx = ParsingResult iloc str in
-      let exc1 = Qerror name ctx exc in
+      let exc1 = Qerror name str ctx exc in
       Ploc.raise loc exc1
-  | Ploc.Exc _ (Qerror _ _ _ as exc) ->
+  | Ploc.Exc _ (Qerror _ _ _ _ as exc) ->
       Ploc.raise loc exc
   | Ploc.Exc iloc exc ->
       let ctx = ParsingResult iloc str in
-      let exc1 = Qerror name ctx exc in
+      let exc1 = Qerror name str ctx exc in
       Ploc.raise loc exc1 ]
 ;
 
@@ -174,7 +244,7 @@ value handle_quotation loc proj proj2 in_expr entry reloc (name, str) =
   in
   let expander =
     try Quotation.find name with exc ->
-      let exc1 = Qerror name Finding exc in
+      let exc1 = Qerror name str Finding exc in
       raise (Ploc.Exc (Ploc.sub loc 0 shift) exc1)
   in
   let ast =
@@ -241,16 +311,16 @@ value string_of_loc fname line bp ep =
       sprintf "File \"%s\", line %d, characters %d-%d:\n" fname line bp ep ]
 ;
 
-value report_quotation_error name ctx = do {
+value pp_report_quotation_error pps name str ctx = do {
   let name = if name = "" then Quotation.default.val else name in
-  Format.print_flush ();
-  Format.open_hovbox 2;
-  eprintf "While %s \"%s\":"
+  Format.pp_print_flush pps ();
+  Format.pp_open_hovbox pps 2;
+  eprintf "While %s \"%s\" for string \"%s\":"
     (match ctx with
      [ Finding -> "finding quotation"
      | Expanding -> "expanding quotation"
      | ParsingResult _ _ -> "parsing result of quotation" ])
-    name;
+    name str;
   match ctx with
   [ ParsingResult loc str ->
       match quotation_dump_file.val with
@@ -283,9 +353,9 @@ value report_quotation_error name ctx = do {
   | _ -> do { eprintf "\n"; flush stderr } ]
 };
 
-value print_format str = do {
-  let rec flush ini cnt =
-    if cnt > ini then Format.print_string (String.sub str ini (cnt - ini))
+value pp_print_format pps str = do {
+  let flush ini cnt =
+    if cnt > ini then Format.pp_print_string pps (String.sub str ini (cnt - ini))
     else ()
   in
   let rec loop ini cnt =
@@ -294,107 +364,101 @@ value print_format str = do {
       match str.[cnt] with
       [ '\n' -> do {
           flush ini cnt;
-          Format.close_box ();
-          Format.force_newline ();
-          Format.open_box 2;
+          Format.pp_close_box pps ();
+          Format.pp_force_newline pps ();
+          Format.pp_open_box pps 2;
           loop (cnt + 1) (cnt + 1)
         }
       | ' ' -> do {
           flush ini cnt;
-          Format.print_space ();
+          Format.pp_print_space pps ();
           loop (cnt + 1) (cnt + 1)
         }
       | _ -> loop ini (cnt + 1) ]
   in
-  Format.open_box 2;
+  Format.pp_open_box pps 2;
   loop 0 0;
-  Format.close_box ()
+  Format.pp_close_box pps ()
 };
 
-value print_file_failed file line char = do {
-  Format.print_string ", file \"";
-  Format.print_string file;
-  Format.print_string "\", line ";
-  Format.print_int line;
-  Format.print_string ", char ";
-  Format.print_int char
+value pp_print_file_failed pps file line char = do {
+  Format.pp_print_string pps ", file \"";
+  Format.pp_print_string pps file;
+  Format.pp_print_string pps "\", line ";
+  Format.pp_print_int pps line;
+  Format.pp_print_string pps ", char ";
+  Format.pp_print_int pps char
 };
 
-value print_exn =
+value pp_print_exn pps =
   fun
-  [ Out_of_memory -> Format.print_string "Out of memory\n"
+  [ Out_of_memory -> Format.pp_print_string pps "Out of memory\n"
   | Assert_failure (file, line, char) -> do {
-      Format.print_string "Assertion failed";
-      print_file_failed file line char
+      Format.pp_print_string pps "Assertion failed";
+      pp_print_file_failed pps file line char
     }
   | Match_failure (file, line, char) -> do {
-      Format.print_string "Pattern matching failed";
-      print_file_failed file line char
+      Format.pp_print_string pps "Pattern matching failed";
+      pp_print_file_failed pps file line char
     }
   | Stream.Error str ->
-      if str = "" then Format.print_string "Parse error"
-      else print_format ("Parse error: " ^ str)
-  | Stream.Failure -> Format.print_string "Parse failure"
+      if str = "" then Format.pp_print_string pps "Parse error"
+      else pp_print_format pps ("Parse error: " ^ str)
+  | Stream.Failure -> Format.pp_print_string pps "Parse failure"
   | Plexing.Error str -> do {
-      Format.print_string "Lexing error";
+      Format.pp_print_string pps "Lexing error";
       if str <> "" then do {
-        Format.print_string ": ";
-        Format.print_string str
+        Format.pp_print_string pps ": ";
+        Format.pp_print_string pps str
       }
-      else Format.print_string ".";
+      else Format.pp_print_string pps ".";
     }
   | Failure str -> do {
-      Format.print_string "Failure: ";
-      Format.print_string str
+      Format.pp_print_string pps "Failure: ";
+      Format.pp_print_string pps str
     }
   | Invalid_argument str -> do {
-      Format.print_string "Invalid argument: ";
-      Format.print_string str
+      Format.pp_print_string pps "Invalid argument: ";
+      Format.pp_print_string pps str
     }
   | Sys_error msg -> do {
-      Format.print_string "I/O error: ";
-      Format.print_string msg
+      Format.pp_print_string pps "I/O error: ";
+      Format.pp_print_string pps msg
     }
   | x -> do {
-      Format.print_string "Uncaught exception: ";
-      Format.print_string
-        (Obj.magic (Obj.field (Obj.field (Obj.repr x) 0) 0));
-      if Obj.size (Obj.repr x) > 1 then do {
-        Format.print_string " (";
-        for i = 1 to Obj.size (Obj.repr x) - 1 do {
-          if i > 1 then Format.print_string ", " else ();
-          let arg = Obj.field (Obj.repr x) i in
-          if not (Obj.is_block arg) then
-            Format.print_int (Obj.magic arg : int)
-          else if Obj.tag arg = Obj.tag (Obj.repr "a") then do {
-            Format.print_char '"';
-            Format.print_string (Obj.magic arg : string);
-            Format.print_char '"'
-          }
-          else Format.print_char '_';
-        };
-        Format.print_char ')'
-      }
-      else ()
+      Format.pp_print_string pps "Uncaught exception: ";
+      Format.pp_print_string pps (Printexc.to_string x);
     } ]
 ;
 
-value report_error exn =
+value pp_report_error pps exn =
   match exn with
-  [ Qerror name Finding Not_found -> do {
+  [ Qerror name str Finding Not_found -> do {
       let name = if name = "" then Quotation.default.val else name in
-      Format.print_flush ();
-      Format.open_hovbox 2;
-      Format.printf "Unbound quotation: \"%s\"" name;
-      Format.close_box ()
+      Format.pp_print_flush pps ();
+      Format.pp_open_hovbox pps 2;
+      Format.fprintf pps "Unbound quotation: \"%s\" for string \"%s\"" name str;
+      Format.pp_close_box pps ()
     }
-  | Qerror name ctx exn -> do {
-      report_quotation_error name ctx;
-      print_exn exn
+  | Qerror name str ctx exn -> do {
+      pp_report_quotation_error pps name str ctx;
+      pp_print_exn pps exn
     }
-  | e -> print_exn exn ]
+  | e -> pp_print_exn pps exn ]
 ;
 
+value report_error exn = pp_report_error Format.std_formatter exn ;
+
+Printexc.register_printer (fun [
+    (Qerror _ _ _ _) as exn ->
+    let b = Buffer.create 23 in
+    let pps = Format.formatter_of_buffer b in do {
+      Format.fprintf pps "%a%!" pp_report_error exn ;
+      Some (Buffer.contents b)
+    }
+  | _ -> None
+]) ;
+
 value no_constructors_arity = Prtools.no_constructors_arity;
 
 value arg_spec_list_ref = ref [];
@@ -409,11 +473,12 @@ value undef x = ref (fun _ -> failwith x);
 value print_interf = undef "no printer";
 value print_implem = undef "no printer";
 
-value pr_expr = Eprinter.make "expr";
+value pr_expr = Eprinter.make ~{fail=Pp_debug.Pp_MLast.show_expr} "expr";
 value pr_patt = Eprinter.make "patt";
 value pr_ctyp = Eprinter.make "type";
 value pr_str_item = Eprinter.make "str_item";
 value pr_sig_item = Eprinter.make "sig_item";
+value pr_longident = Eprinter.make "longident";
 value pr_module_expr = Eprinter.make "module_expr";
 value pr_module_type = Eprinter.make "module_type";
 value pr_class_sig_item = Eprinter.make "class_sig_item";
@@ -424,7 +489,7 @@ value pr_expr_fun_args = ref Extfun.empty;
 
 value flag_comments_in_phrases = ref True;
 value flag_equilibrate_cases = ref False;
-value flag_compatible_old_versions_of_ocaml = ref False;
+value flag_expand_letop_syntax = ref False ;
 
 value inter_phrases = ref None;
 
@@ -497,6 +562,15 @@ value unvala x =
     | Ploc.VaAnt a -> failwith ("unexpected antiquotation value " ^ a) ]
   END
 ;
+value vala_it f x =
+  IFNDEF STRICT THEN ignore(f x)
+  ELSE
+    match x with
+    [ Ploc.VaVal x ->  ignore (f x)
+    | Ploc.VaAnt a -> () ]
+  END
+;
+
 value vala_map f x =
   IFNDEF STRICT THEN f x
   ELSE
