@@ -61,8 +61,10 @@ do {
   Grammar.Unsafe.clear_entry class_str_item
 };
 
-Pcaml.parse_interf.val := Grammar.Entry.parse interf;
-Pcaml.parse_implem.val := Grammar.Entry.parse implem;
+Pcaml.(set_ast_parse transduce_interf (Grammar.Entry.parse interf));
+Pcaml.(set_ast_parse transduce_implem (Grammar.Entry.parse implem));
+Pcaml.(set_ast_parse transduce_top_phrase (Grammar.Entry.parse top_phrase));
+Pcaml.(set_ast_parse transduce_use_file (Grammar.Entry.parse use_file));
 
 Pcaml.add_option "-ignloaddir"
   (Arg.Unit (fun _ → add_directive "load" (fun _ → ())))
@@ -160,7 +162,8 @@ value operator_rparen =
 
 value check_not_part_of_patt_f strm =
   let matchers = [
-    (2, fun [ [("LIDENT", _); tok :: _] -> Some tok | _ -> None ])
+    (1, fun [ [(("", "[") as tok) :: _] -> Some tok | _ -> None ])
+  ; (2, fun [ [("LIDENT", _); tok :: _] -> Some tok | _ -> None ])
   ; (4, fun [ [("", "("); ("", s); ("", ")"); tok :: _] when is_special_op s -> Some tok | _ -> None ])
   ; (6, fun [
               [("", "("); ("", s); ("", "("); ("", ")"); ("", ")"); tok :: _] when is_special_op s -> Some tok
@@ -199,7 +202,7 @@ value check_not_part_of_patt_f strm =
   ] in
   let tok = crec 1 matchers in
   match tok with
-    [ ("", "," | "as" | "|" | "::") -> raise Stream.Failure
+    [ ("", "," | "as" | "|" | "::" | "[") -> raise Stream.Failure
     | _ -> () ]
 ;
 
@@ -1854,6 +1857,7 @@ EXTEND
   ;
   top_phrase:
     [ [ ph = phrase → Some ph
+      | ";" -> Some <:str_item< declare end >>
       | EOI → None ] ]
   ;
   use_file:
