@@ -562,6 +562,22 @@ let ocaml_pexp_variant =
   Some (pexp_variant_pat, pexp_variant)
 ;;
 
+let ocaml_value_binding_constraint loc vb (sl, ct) =
+  if sl = [] then
+    let e = vb.pvb_expr in
+    let e = ocaml_mkexp loc (ocaml_pexp_constraint e (Some ct) None) in
+    {vb with pvb_expr = e}
+  else
+    failwith
+      "ocaml_value_binding_constraint: Only available in Ocaml versions >= 5.1.0"
+;;
+
+let ocaml_value_binding_coerce loc vb (ct_opt, ct2) =
+  let e = vb.pvb_expr in
+  let e = ocaml_mkexp loc (ocaml_pexp_constraint e ct_opt (Some ct2)) in
+  {vb with pvb_expr = e}
+;;
+
 let ocaml_value_binding ?(item_attributes = []) loc p e =
   let p =
     match p with
@@ -585,6 +601,7 @@ let ocaml_ppat_alias p i iloc = Ppat_alias (p, mkloc iloc i);;
 let ocaml_ppat_array = Some (fun pl -> Ppat_array pl);;
 
 let ocaml_ppat_construct loc li po chk_arity =
+  let po = option_map (fun (_, p) -> p) po in
   Ppat_construct (mkloc loc li, po)
 ;;
 
@@ -671,11 +688,7 @@ let ocaml_psig_modtype ?(item_attributes = []) loc s mto =
 ;;
 
 let ocaml_psig_modtypesubst ?(item_attributes = []) loc s mto =
-  let pmtd =
-    {pmtd_name = mkloc loc s; pmtd_type = mto;
-     pmtd_attributes = item_attributes; pmtd_loc = loc}
-  in
-  Psig_modtypesubst pmtd
+  failwith "no 'module type' substitution in this version of ocaml"
 ;;
 
 let ocaml_psig_open ?(item_attributes = []) loc li =
@@ -927,12 +940,12 @@ let ocaml_ptop_dir loc s da =
 
   let (ocaml_pwith_modtype :
  (Location.t -> Longident.t -> module_type -> with_constraint) option) =
-  Some (fun loc li mt -> Pwith_modtype (mkloc loc li, mt))
+  None
 ;;
 
 let (ocaml_pwith_modtypesubst :
  (Location.t -> Longident.t -> module_type -> with_constraint) option) =
-  Some (fun loc li mt -> Pwith_modtypesubst (mkloc loc li, mt))
+  None
 ;;
 
 let ocaml_pwith_modsubst =
