@@ -1,6 +1,6 @@
 (* camlp5r *)
 (* parserify.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 (* #load "q_MLast.cmo" *)
 
@@ -9,20 +9,11 @@
 open Pretty;;
 open Prtools;;
 open Versdep;;
+open Exparser;;
 
-type spat_comp =
-    SpTrm of MLast.loc * MLast.patt * MLast.expr option MLast.v
-  | SpNtr of MLast.loc * MLast.patt * MLast.expr
-  | SpLet of MLast.loc * MLast.patt * MLast.expr
-  | SpLhd of MLast.loc * MLast.patt list list
-  | SpStr of MLast.loc * MLast.patt
-;;
-
-type spat_comp_opt =
-    SpoNoth
-  | SpoBang
-  | SpoQues of MLast.expr
-;;
+type spat_comp = Exparser.spat_comp;;
+type spat_comp_opt = Exparser.spat_comp_opt;;
+type spat_parser_ast = Exparser.spat_parser_ast;;
 
 (* Rebuilding syntax tree *)
 
@@ -39,25 +30,25 @@ let rec handle_failure e =
         pel
   | MLast.ExApp (_, MLast.ExLid (_, "raise"), e) ->
       begin match e with
-        MLast.ExAcc
-          (_, MLast.ExUid (_, "Stream"), MLast.ExUid (_, "Failure")) ->
+        MLast.ExLong
+          (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure")) ->
           false
       | _ -> true
       end
   | MLast.ExTry
       (_, te,
-       [MLast.PaAcc
-          (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
+       [MLast.PaLong
+          (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
         None, e]) ->
       handle_failure e
   | MLast.ExApp (_, f, e) ->
       no_raising_failure_fun f && handle_failure f && handle_failure e
-  | MLast.ExLid (_, _) | MLast.ExUid (_, _) -> true
+  | MLast.ExLid (_, _) | MLast.ExLong (_, MLast.LiUid (_, _)) -> true
   | _ -> false
 and no_raising_failure_fun =
   function
     MLast.ExApp (_, x, y) -> no_raising_failure_fun x && handle_failure y
-  | MLast.ExUid (_, _) -> true
+  | MLast.ExLong (_, MLast.LiUid (_, _)) -> true
   | _ -> false
 ;;
 
@@ -82,26 +73,32 @@ let rec simpl =
     MLast.ExMat
       (_,
        MLast.ExTry
-         (_, MLast.ExApp (_, MLast.ExUid (_, "Some"), f),
-          [MLast.PaAcc
-             (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
-           None, MLast.ExUid (_, "None")]),
-       [MLast.PaApp (_, MLast.PaUid (_, "Some"), MLast.PaLid (_, s1)), None,
-        MLast.ExLid (_, s2);
+         (_, MLast.ExApp (_, MLast.ExLong (_, MLast.LiUid (_, "Some")), f),
+          [MLast.PaLong
+             (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
+           None, MLast.ExLong (_, MLast.LiUid (_, "None"))]),
+       [MLast.PaApp
+          (_, MLast.PaLong (_, MLast.LiUid (_, "Some"), []),
+           MLast.PaLid (_, s1)),
+        None, MLast.ExLid (_, s2);
         MLast.PaAny _, None,
         MLast.ExApp
           (_, MLast.ExLid (_, "raise"),
-           MLast.ExAcc
-             (_, MLast.ExUid (_, "Stream"),
-              MLast.ExUid (_, "Failure")))]) as e ->
+           MLast.ExLong
+             (_,
+              MLast.LiAcc
+                (_, MLast.LiUid (_, "Stream"), "Failure")))]) as e ->
       if s1 = s2 then f else e
   | MLast.ExMat
       (_, e,
-       [MLast.PaApp (_, MLast.PaUid (_, "Some"), p1), None, e1;
+       [MLast.PaApp (_, MLast.PaLong (_, MLast.LiUid (_, "Some"), []), p1),
+        None, e1;
         MLast.PaAny _, None, e2]) ->
       MLast.ExMat
         (loc, e,
-         [MLast.PaApp (loc, MLast.PaUid (loc, "Some"), p1), None, e1;
+         [MLast.PaApp
+            (loc, MLast.PaLong (loc, MLast.LiUid (loc, "Some"), []), p1),
+          None, e1;
           MLast.PaAny loc, None, simpl e2])
   | e -> e
 ;;
@@ -113,50 +110,34 @@ let rec unstream_pattern_kont =
        [p,
         MLast.ExTry
           (_, f,
-           [MLast.PaAcc
-              (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
+           [MLast.PaLong
+              (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
             None,
             MLast.ExApp
               (_, MLast.ExLid (_, "raise"),
                MLast.ExApp
                  (_,
-                  MLast.ExAcc
-                    (_, MLast.ExUid (_, "Stream"), MLast.ExUid (_, "Error")),
-                  e2))])],
-       e) ->
-      let f =
-        match f with
-          MLast.ExApp (_, f, MLast.ExLid (_, "strm__")) -> f
-        | _ ->
-            MLast.ExFun
-              (loc,
-               [MLast.PaTyc
-                  (loc, MLast.PaLid (loc, "strm__"),
-                   MLast.TyApp
-                     (loc,
-                      MLast.TyAcc
-                        (loc, MLast.TyUid (loc, "Stream"),
-                         MLast.TyLid (loc, "t")),
-                      MLast.TyAny loc)),
-                None, f])
-      in
-      let (sp, epo, e) = unstream_pattern_kont e in
-      (SpNtr (loc, p, f), err e2) :: sp, epo, e
-  | MLast.ExMat
+                  MLast.ExLong
+                    (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Error")),
+                  e2))]),
+        []],
+       e) |
+    MLast.ExMat
       (_,
        MLast.ExTry
-         (_, MLast.ExApp (_, MLast.ExUid (_, "Some"), f),
-          [MLast.PaAcc
-             (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
-           None, MLast.ExUid (_, "None")]),
-       [MLast.PaApp (_, MLast.PaUid (_, "Some"), p), None, e;
+         (_, MLast.ExApp (_, MLast.ExLong (_, MLast.LiUid (_, "Some")), f),
+          [MLast.PaLong
+             (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
+           None, MLast.ExLong (_, MLast.LiUid (_, "None"))]),
+       [MLast.PaApp (_, MLast.PaLong (_, MLast.LiUid (_, "Some"), []), p),
+        None, e;
         MLast.PaAny _, None,
         MLast.ExApp
           (_, MLast.ExLid (_, "raise"),
            MLast.ExApp
              (_,
-              MLast.ExAcc
-                (_, MLast.ExUid (_, "Stream"), MLast.ExUid (_, "Error")),
+              MLast.ExLong
+                (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Error")),
               e2))]) ->
       let f =
         match f with
@@ -168,9 +149,7 @@ let rec unstream_pattern_kont =
                   (loc, MLast.PaLid (loc, "strm__"),
                    MLast.TyApp
                      (loc,
-                      MLast.TyAcc
-                        (loc, MLast.TyUid (loc, "Stream"),
-                         MLast.TyLid (loc, "t")),
+                      MLast.TyAcc (loc, MLast.LiUid (loc, "Stream"), "t"),
                       MLast.TyAny loc)),
                 None, f])
       in
@@ -181,12 +160,14 @@ let rec unstream_pattern_kont =
        [MLast.PaLid (_, p),
         MLast.ExApp
           (_,
-           MLast.ExAcc
-             (_, MLast.ExUid (_, "Stream"), MLast.ExLid (_, "count")),
-           MLast.ExLid (_, "strm__"))],
+           MLast.ExFle
+             (_, MLast.ExLong (_, MLast.LiUid (_, "Stream")),
+              (None, "count")),
+           MLast.ExLid (_, "strm__")),
+        []],
        e) ->
-      [], Some p, e
-  | MLast.ExLet (_, false, [p, MLast.ExLid (_, "strm__")], e) ->
+      [], Some (MLast.PaLid (loc, p)), e
+  | MLast.ExLet (_, false, [p, MLast.ExLid (_, "strm__"), []], e) ->
       let (sp, epo, e) = unstream_pattern_kont e in
       (SpStr (loc, p), SpoNoth) :: sp, epo, e
   | MLast.ExMat
@@ -194,19 +175,22 @@ let rec unstream_pattern_kont =
        MLast.ExTry
          (_,
           MLast.ExApp
-            (_, MLast.ExUid (_, "Some"),
+            (_, MLast.ExLong (_, MLast.LiUid (_, "Some")),
              MLast.ExApp (_, f, MLast.ExLid (_, "strm__"))),
-          [MLast.PaAcc
-             (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
-           None, MLast.ExUid (_, "None")]),
-       [MLast.PaApp (_, MLast.PaUid (_, "Some"), MLast.PaLid (_, s1)), None,
-        MLast.ExLid (_, s2);
+          [MLast.PaLong
+             (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
+           None, MLast.ExLong (_, MLast.LiUid (_, "None"))]),
+       [MLast.PaApp
+          (_, MLast.PaLong (_, MLast.LiUid (_, "Some"), []),
+           MLast.PaLid (_, s1)),
+        None, MLast.ExLid (_, s2);
         MLast.PaAny _, None,
         MLast.ExApp
           (_, MLast.ExLid (_, "raise"),
-           MLast.ExAcc
-             (_, MLast.ExUid (_, "Stream"),
-              MLast.ExUid (_, "Failure")))]) as e ->
+           MLast.ExLong
+             (_,
+              MLast.LiAcc
+                (_, MLast.LiUid (_, "Stream"), "Failure")))]) as e ->
       if s1 = s2 then
         [SpNtr (loc, MLast.PaLid (loc, "a"), f), SpoBang], None,
         MLast.ExLid (loc, "a")
@@ -214,20 +198,22 @@ let rec unstream_pattern_kont =
   | MLast.ExMat
       (_,
        MLast.ExTry
-         (_, MLast.ExApp (_, MLast.ExUid (_, "Some"), e1),
-          [MLast.PaAcc
-             (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
-           None, MLast.ExUid (_, "None")]),
-       [MLast.PaApp (_, MLast.PaUid (_, "Some"), MLast.PaLid (_, s)), None,
-        e2;
+         (_, MLast.ExApp (_, MLast.ExLong (_, MLast.LiUid (_, "Some")), e1),
+          [MLast.PaLong
+             (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
+           None, MLast.ExLong (_, MLast.LiUid (_, "None"))]),
+       [MLast.PaApp
+          (_, MLast.PaLong (_, MLast.LiUid (_, "Some"), []),
+           MLast.PaLid (_, s)),
+        None, e2;
         MLast.PaAny _, None,
         MLast.ExApp
           (_, MLast.ExLid (_, "raise"),
-           MLast.ExAcc
-             (_, MLast.ExUid (_, "Stream"), MLast.ExUid (_, "Failure")))]) ->
-      let e = MLast.ExLet (loc, false, [MLast.PaLid (loc, s), e1], e2) in
+           MLast.ExLong
+             (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure")))]) ->
+      let e = MLast.ExLet (loc, false, [MLast.PaLid (loc, s), e1, []], e2) in
       unstream_pattern_kont e
-  | MLast.ExLet (_, false, [p, e1], e2) as ge ->
+  | MLast.ExLet (_, false, [p, e1, []], e2) as ge ->
       if contains_strm__ e1 then
         let (f, err) =
           match e1 with
@@ -240,9 +226,7 @@ let rec unstream_pattern_kont =
                       (loc, MLast.PaLid (loc, "strm__"),
                        MLast.TyApp
                          (loc,
-                          MLast.TyAcc
-                            (loc, MLast.TyUid (loc, "Stream"),
-                             MLast.TyLid (loc, "t")),
+                          MLast.TyAcc (loc, MLast.LiUid (loc, "Stream"), "t"),
                           MLast.TyAny loc)),
                     None, e1])
               in
@@ -259,15 +243,18 @@ let rec unstream_pattern_kont =
       (_,
        MLast.ExApp
          (_,
-          MLast.ExAcc (_, MLast.ExUid (_, "Stream"), MLast.ExLid (_, "peek")),
+          MLast.ExFle
+            (_, MLast.ExLong (_, MLast.LiUid (_, "Stream")), (None, "peek")),
           MLast.ExLid (_, "strm__")),
-       [MLast.PaApp (_, MLast.PaUid (_, "Some"), p), None,
+       [MLast.PaApp (_, MLast.PaLong (_, MLast.LiUid (_, "Some"), []), p),
+        None,
         MLast.ExSeq
           (_,
            [MLast.ExApp
               (_,
-               MLast.ExAcc
-                 (_, MLast.ExUid (_, "Stream"), MLast.ExLid (_, "junk")),
+               MLast.ExFle
+                 (_, MLast.ExLong (_, MLast.LiUid (_, "Stream")),
+                  (None, "junk")),
                MLast.ExLid (_, "strm__"));
             e]);
         MLast.PaAny _, None,
@@ -275,8 +262,8 @@ let rec unstream_pattern_kont =
           (_, MLast.ExLid (_, "raise"),
            MLast.ExApp
              (_,
-              MLast.ExAcc
-                (_, MLast.ExUid (_, "Stream"), MLast.ExUid (_, "Error")),
+              MLast.ExLong
+                (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Error")),
               e2))]) ->
       let (sp, epo, e) = unstream_pattern_kont e in
       let sp = (SpTrm (loc, p, None), err e2) :: sp in sp, epo, e
@@ -284,7 +271,8 @@ let rec unstream_pattern_kont =
       (_,
        MLast.ExApp
          (_,
-          MLast.ExAcc (_, MLast.ExUid (_, "Stream"), MLast.ExLid (_, "peek")),
+          MLast.ExFle
+            (_, MLast.ExLong (_, MLast.LiUid (_, "Stream")), (None, "peek")),
           MLast.ExLid (_, "strm__")),
        _) |
     MLast.ExMat
@@ -292,12 +280,13 @@ let rec unstream_pattern_kont =
        MLast.ExTry
          (_,
           MLast.ExApp
-            (_, MLast.ExUid (_, "Some"),
+            (_, MLast.ExLong (_, MLast.LiUid (_, "Some")),
              MLast.ExApp (_, _, MLast.ExLid (_, "strm__"))),
-          [MLast.PaAcc
-             (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
-           None, MLast.ExUid (_, "None")]),
-       [MLast.PaApp (_, MLast.PaUid (_, "Some"), _), None, _;
+          [MLast.PaLong
+             (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
+           None, MLast.ExLong (_, MLast.LiUid (_, "None"))]),
+       [MLast.PaApp (_, MLast.PaLong (_, MLast.LiUid (_, "Some"), []), _),
+        None, _;
         MLast.PaAny _, None, _]) as ge ->
       let f =
         MLast.ExFun
@@ -305,10 +294,7 @@ let rec unstream_pattern_kont =
            [MLast.PaTyc
               (loc, MLast.PaLid (loc, "strm__"),
                MLast.TyApp
-                 (loc,
-                  MLast.TyAcc
-                    (loc, MLast.TyUid (loc, "Stream"),
-                     MLast.TyLid (loc, "t")),
+                 (loc, MLast.TyAcc (loc, MLast.LiUid (loc, "Stream"), "t"),
                   MLast.TyAny loc)),
             None, ge])
       in
@@ -317,29 +303,29 @@ let rec unstream_pattern_kont =
       MLast.ExLid (loc, "a")
   | MLast.ExTry
       (_, MLast.ExApp (_, f, MLast.ExLid (_, "strm__")),
-       [MLast.PaAcc
-          (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
+       [MLast.PaLong
+          (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
         None,
         MLast.ExApp
           (_, MLast.ExLid (_, "raise"),
            MLast.ExApp
              (_,
-              MLast.ExAcc
-                (_, MLast.ExUid (_, "Stream"), MLast.ExUid (_, "Error")),
+              MLast.ExLong
+                (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Error")),
               e))]) ->
       [SpNtr (loc, MLast.PaLid (loc, "a"), f), err e], None,
       MLast.ExLid (loc, "a")
   | MLast.ExTry
       (_, f,
-       [MLast.PaAcc
-          (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
+       [MLast.PaLong
+          (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
         None,
         MLast.ExApp
           (_, MLast.ExLid (_, "raise"),
            MLast.ExApp
              (_,
-              MLast.ExAcc
-                (_, MLast.ExUid (_, "Stream"), MLast.ExUid (_, "Error")),
+              MLast.ExLong
+                (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Error")),
               e))]) ->
       let f =
         MLast.ExFun
@@ -347,10 +333,7 @@ let rec unstream_pattern_kont =
            [MLast.PaTyc
               (loc, MLast.PaLid (loc, "strm__"),
                MLast.TyApp
-                 (loc,
-                  MLast.TyAcc
-                    (loc, MLast.TyUid (loc, "Stream"),
-                     MLast.TyLid (loc, "t")),
+                 (loc, MLast.TyAcc (loc, MLast.LiUid (loc, "Stream"), "t"),
                   MLast.TyAny loc)),
             None, f])
       in
@@ -369,9 +352,10 @@ let rec unparser_cases_list =
        [p,
         MLast.ExTry
           (_, MLast.ExApp (_, f, MLast.ExLid (_, "strm__")),
-           [MLast.PaAcc
-              (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
-            None, MLast.ExApp (_, MLast.ExLid (_, "raise"), e2)])],
+           [MLast.PaLong
+              (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
+            None, MLast.ExApp (_, MLast.ExLid (_, "raise"), e2)]),
+        []],
        e1) ->
       let spe1 =
         let (sp, epo, e) = unstream_pattern_kont e1 in
@@ -386,36 +370,41 @@ let rec unparser_cases_list =
        [MLast.PaLid (_, p),
         MLast.ExApp
           (_,
-           MLast.ExAcc
-             (_, MLast.ExUid (_, "Stream"), MLast.ExLid (_, "count")),
-           MLast.ExLid (_, "strm__"))],
+           MLast.ExFle
+             (_, MLast.ExLong (_, MLast.LiUid (_, "Stream")),
+              (None, "count")),
+           MLast.ExLid (_, "strm__")),
+        []],
        e) ->
-      [[], Some p, e]
+      [[], Some (MLast.PaLid (loc, p)), e]
   | MLast.ExLet
-      (_, false, [p, MLast.ExApp (_, f, MLast.ExLid (_, "strm__"))], e) ->
+      (_, false, [p, MLast.ExApp (_, f, MLast.ExLid (_, "strm__")), []], e) ->
       let (sp, epo, e) = unstream_pattern_kont e in
       [(SpNtr (loc, p, f), SpoNoth) :: sp, epo, e]
-  | MLast.ExLet (_, false, [p, MLast.ExLid (_, "strm__")], e) ->
+  | MLast.ExLet (_, false, [p, MLast.ExLid (_, "strm__"), []], e) ->
       let (sp, epo, e) = unstream_pattern_kont e in
       [(SpStr (loc, p), SpoNoth) :: sp, epo, e]
   | MLast.ExMat
       (_,
        MLast.ExApp
          (_,
-          MLast.ExAcc (_, MLast.ExUid (_, "Stream"), MLast.ExLid (_, "peek")),
+          MLast.ExFle
+            (_, MLast.ExLong (_, MLast.LiUid (_, "Stream")), (None, "peek")),
           MLast.ExLid (_, "strm__")),
        pel) as ge ->
       let rec loop rev_spel =
         function
           [MLast.PaAny _, None, e] ->
             list_rev_append rev_spel (unparser_cases_list e)
-        | (MLast.PaApp (_, MLast.PaUid (_, "Some"), p), eo,
+        | (MLast.PaApp (_, MLast.PaLong (_, MLast.LiUid (_, "Some"), []), p),
+           eo,
            MLast.ExSeq
              (_,
               [MLast.ExApp
                  (_,
-                  MLast.ExAcc
-                    (_, MLast.ExUid (_, "Stream"), MLast.ExLid (_, "junk")),
+                  MLast.ExFle
+                    (_, MLast.ExLong (_, MLast.LiUid (_, "Stream")),
+                     (None, "junk")),
                   MLast.ExLid (_, "strm__"));
                e])) ::
           pel ->
@@ -432,12 +421,13 @@ let rec unparser_cases_list =
        MLast.ExTry
          (_,
           MLast.ExApp
-            (_, MLast.ExUid (_, "Some"),
+            (_, MLast.ExLong (_, MLast.LiUid (_, "Some")),
              MLast.ExApp (_, f, MLast.ExLid (_, "strm__"))),
-          [MLast.PaAcc
-             (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
-           None, MLast.ExUid (_, "None")]),
-       [MLast.PaApp (_, MLast.PaUid (_, "Some"), p1), None, e1;
+          [MLast.PaLong
+             (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
+           None, MLast.ExLong (_, MLast.LiUid (_, "None"))]),
+       [MLast.PaApp (_, MLast.PaLong (_, MLast.LiUid (_, "Some"), []), p1),
+        None, e1;
         MLast.PaAny _, None, e2]) ->
       let spe =
         let (sp, epo, e) = unstream_pattern_kont e1 in
@@ -447,29 +437,35 @@ let rec unparser_cases_list =
   | MLast.ExMat
       (_,
        MLast.ExTry
-         (_, MLast.ExApp (_, MLast.ExUid (_, "Some"), f),
-          [MLast.PaAcc
-             (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
-           None, MLast.ExUid (_, "None")]),
-       [MLast.PaApp (_, MLast.PaUid (_, "Some"), p1), None, e1;
+         (_, MLast.ExApp (_, MLast.ExLong (_, MLast.LiUid (_, "Some")), f),
+          [MLast.PaLong
+             (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
+           None, MLast.ExLong (_, MLast.LiUid (_, "None"))]),
+       [MLast.PaApp (_, MLast.PaLong (_, MLast.LiUid (_, "Some"), []), p1),
+        None, e1;
         MLast.PaAny _, None, e2]) ->
       let e =
         MLast.ExMat
           (loc,
            MLast.ExTry
-             (loc, MLast.ExApp (loc, MLast.ExUid (loc, "Some"), simpl f),
-              [MLast.PaAcc
-                 (loc, MLast.PaUid (loc, "Stream"),
-                  MLast.PaUid (loc, "Failure")),
-               None, MLast.ExUid (loc, "None")]),
-           [MLast.PaApp (loc, MLast.PaUid (loc, "Some"), p1), None, simpl e1;
+             (loc,
+              MLast.ExApp
+                (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Some")), simpl f),
+              [MLast.PaLong
+                 (loc,
+                  MLast.LiAcc (loc, MLast.LiUid (loc, "Stream"), "Failure"),
+                  []),
+               None, MLast.ExLong (loc, MLast.LiUid (loc, "None"))]),
+           [MLast.PaApp
+              (loc, MLast.PaLong (loc, MLast.LiUid (loc, "Some"), []), p1),
+            None, simpl e1;
             MLast.PaAny loc, None, simpl e2])
       in
       [[], None, e]
   | MLast.ExTry
       (_, MLast.ExApp (_, f, MLast.ExLid (_, "strm__")),
-       [MLast.PaAcc
-          (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
+       [MLast.PaLong
+          (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
         None, e]) ->
       let spe =
         [SpNtr (loc, MLast.PaLid (loc, "a"), f), SpoNoth], None,
@@ -479,20 +475,22 @@ let rec unparser_cases_list =
   | MLast.ExTry
       (_,
        MLast.ExApp
-         (_, MLast.ExUid (_, "Some"),
+         (_, MLast.ExLong (_, MLast.LiUid (_, "Some")),
           MLast.ExApp (_, f, MLast.ExLid (_, "strm__"))),
-       [MLast.PaAcc
-          (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
+       [MLast.PaLong
+          (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
         None, e]) ->
       let spe =
         [SpNtr (loc, MLast.PaLid (loc, "a"), f), SpoNoth], None,
-        MLast.ExApp (loc, MLast.ExUid (loc, "Some"), MLast.ExLid (loc, "a"))
+        MLast.ExApp
+          (loc, MLast.ExLong (loc, MLast.LiUid (loc, "Some")),
+           MLast.ExLid (loc, "a"))
       in
       let spel = unparser_cases_list e in spe :: spel
   | MLast.ExTry
       (_, f,
-       [MLast.PaAcc
-          (_, MLast.PaUid (_, "Stream"), MLast.PaUid (_, "Failure")),
+       [MLast.PaLong
+          (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"), []),
         None, e]) ->
       let f =
         MLast.ExFun
@@ -500,10 +498,7 @@ let rec unparser_cases_list =
            [MLast.PaTyc
               (loc, MLast.PaLid (loc, "strm__"),
                MLast.TyApp
-                 (loc,
-                  MLast.TyAcc
-                    (loc, MLast.TyUid (loc, "Stream"),
-                     MLast.TyLid (loc, "t")),
+                 (loc, MLast.TyAcc (loc, MLast.LiUid (loc, "Stream"), "t"),
                   MLast.TyAny loc)),
             None, f])
       in
@@ -520,8 +515,8 @@ let rec unparser_cases_list =
       [spe]
   | MLast.ExApp
       (_, MLast.ExLid (_, "raise"),
-       MLast.ExAcc
-         (_, MLast.ExUid (_, "Stream"), MLast.ExUid (_, "Failure"))) ->
+       MLast.ExLong
+         (_, MLast.LiAcc (_, MLast.LiUid (_, "Stream"), "Failure"))) ->
       []
   | e -> [[], None, e]
 ;;
@@ -534,11 +529,13 @@ let unparser_body e =
          [MLast.PaLid (_, bp),
           MLast.ExApp
             (_,
-             MLast.ExAcc
-               (_, MLast.ExUid (_, "Stream"), MLast.ExLid (_, "count")),
-             MLast.ExLid (_, strm_n))],
+             MLast.ExFle
+               (_, MLast.ExLong (_, MLast.LiUid (_, "Stream")),
+                (None, "count")),
+             MLast.ExLid (_, strm_n)),
+          []],
          e) ->
-        Some bp, e
+        Some (MLast.PaLid (loc, bp)), e
     | _ -> None, e
   in
   let spel = unparser_cases_list e in po, spel
