@@ -1,26 +1,47 @@
 (* camlp5r *)
 (* pcaml.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2019 *)
 
 (* #load "pa_macro.cmo" *)
 (* #load "pa_extend.cmo" *)
 
 open Printf;;
 
-let version = "6.16";;
+let version = "8.02.01";;
 let syntax_name = ref "";;
 
+let ocaml_version =
+  let rec loop i =
+    if i = String.length Versdep.sys_ocaml_version then
+      Versdep.sys_ocaml_version
+    else
+      match Versdep.sys_ocaml_version.[i] with
+        ' ' | '+' | '~' -> String.sub Versdep.sys_ocaml_version 0 i
+      | _ -> loop (i + 1)
+  in
+  loop 0
+;;
+
 let gram =
   Grammar.gcreate
     {Plexing.tok_func = (fun _ -> failwith "no loaded parsing module");
      Plexing.tok_using = (fun _ -> ()); Plexing.tok_removing = (fun _ -> ());
      Plexing.tok_match =
-       (fun _ -> raise (Match_failure ("pcaml.ml", 17, 25)));
+       (fun _ -> raise (Match_failure ("pcaml.ml", 28, 25)));
      Plexing.tok_text = (fun _ -> ""); Plexing.tok_comm = None}
 ;;
 
+(*
+Camlp5 can be parsed with limited or full backtracking:
+Grammar.set_algorithm gram Grammar.Functional;
+Grammar.set_algorithm gram Grammar.Backtracking;
+or without any change in the code, by setting the environment
+variable CAMLP5PARAM to f or b.
+*)
+
 type status = Ploc.t option;;
 
+let attribute_body = Grammar.Entry.create gram "attribute_body";;
 let interf = Grammar.Entry.create gram "interf";;
 let implem = Grammar.Entry.create gram "implem";;
 let top_phrase = Grammar.Entry.create gram "top_phrase";;
@@ -29,7 +50,11 @@ let signature = Grammar.Entry.create gram "signature";;
 let structure = Grammar.Entry.create gram "structure";;
 let sig_item = Grammar.Entry.create gram "sig_item";;
 let str_item = Grammar.Entry.create gram "str_item";;
+let functor_parameter = Grammar.Entry.create gram "functor_parameter";;
 let module_type = Grammar.Entry.create gram "module_type";;
+let longident = Grammar.Entry.create gram "longident";;
+let longident_lident = Grammar.Entry.create gram "longident_lident";;
+let extended_longident = Grammar.Entry.create gram "extended_longident";;
 let module_expr = Grammar.Entry.create gram "module_expr";;
 let expr = Grammar.Entry.create gram "expr";;
 let patt = Grammar.Entry.create gram "patt";;
@@ -37,6 +62,10 @@ let ipatt = Grammar.Entry.create gram "ipatt";;
 let ctyp = Grammar.Entry.create gram "ctyp";;
 let let_binding = Grammar.Entry.create gram "let_binding";;
 let type_decl = Grammar.Entry.create gram "type_declaration";;
+let type_extension = Grammar.Entry.create gram "type_extension";;
+let extension_constructor =
+  Grammar.Entry.create gram "extension_constructor"
+;;
 let match_case = Grammar.Entry.create gram "match_case";;
 let constructor_declaration =
   Grammar.Entry.create gram "constructor_declaration"
@@ -49,9 +78,71 @@ let class_sig_item = Grammar.Entry.create gram "class_sig_item";;
 let class_str_item = Grammar.Entry.create gram "class_str_item";;
 let class_type = Grammar.Entry.create gram "class_type";;
 let class_expr = Grammar.Entry.create gram "class_expr";;
+let class_expr_simple = Grammar.Entry.create gram "class_expr_simple";;
 
-let parse_interf = ref (Grammar.Entry.parse interf);;
-let parse_implem = ref (Grammar.Entry.parse implem);;
+let alg_attribute = Grammar.Entry.create gram "alg_attribute";;
+let alg_attributes = Grammar.Entry.create gram "alg_attributes";;
+let ext_attributes = Grammar.Entry.create gram "ext_attributes";;
+
+type 'a ast_transducer_t =
+  { name : string;
+    parse : (char Stream.t -> 'a) option ref;
+    transform : ('a -> 'a) option ref }
+;;
+
+let set_ast_parse att pf =
+  match !(att.parse) with
+    Some _ ->
+      failwith
+        (Printf.sprintf
+           "Pcaml.set_ast_parse: transducer \"%s\" already has a parse(r)"
+           att.name)
+  | None -> att.parse := Some pf
+;;
+
+let set_ast_transform att tf =
+  match !(att.transform) with
+    Some _ ->
+      failwith
+        (Printf.sprintf
+           "Pcaml.set_ast_transform: transducer \"%s\" already has a tranform(er)"
+           att.name)
+  | None -> att.transform := Some tf
+;;
+
+let transduce att x =
+  let parse =
+    match !(att.parse) with
+      None ->
+        failwith
+          (Printf.sprintf
+             "Pcaml.transduce: transducer \"%s\" has no configured parser"
+             att.name)
+    | Some x -> x
+  in
+  let x = parse x in
+  match !(att.transform) with
+    None -> x
+  | Some f -> f x
+;;
+
+let transduce_interf =
+  {name = "interf"; parse = ref None; transform = ref None}
+;;
+let transduce_implem =
+  {name = "implem"; parse = ref None; transform = ref None}
+;;
+let transduce_top_phrase =
+  {name = "top_phrase"; parse = ref None; transform = ref None}
+;;
+let transduce_use_file =
+  {name = "use_file"; parse = ref None; transform = ref None}
+;;
+
+let parse_interf x = transduce transduce_interf x;;
+let parse_implem x = transduce transduce_implem x;;
+let parse_top_phrase x = transduce transduce_top_phrase x;;
+let parse_use_file x = transduce transduce_use_file x;;
 
 let rec skip_to_eol cs =
   match Stream.peek cs with
@@ -83,7 +174,7 @@ type err_ctx =
   | Expanding
   | ParsingResult of Ploc.t * string
 ;;
-exception Qerror of string * err_ctx * exn;;
+exception Qerror of string * string * err_ctx * exn;;
 
 let quotation_location () =
   match !quotation_loc with
@@ -110,7 +201,7 @@ let expand_quotation gloc expander shift name str =
     try
       try expander str with
         Ploc.Exc (loc, exc) ->
-          let exc1 = Qerror (name, Expanding, exc) in
+          let exc1 = Qerror (name, str, Expanding, exc) in
           let shift = Ploc.first_pos gloc + shift in
           let loc =
             let gloc_line_nb = Ploc.line_nb gloc in
@@ -127,7 +218,8 @@ let expand_quotation gloc expander shift name str =
           in
           raise (Ploc.Exc (loc, exc1))
       | exc ->
-          let exc1 = Qerror (name, Expanding, exc) in Ploc.raise gloc exc1
+          let exc1 = Qerror (name, str, Expanding, exc) in
+          Ploc.raise gloc exc1
     with exn -> restore (); raise exn
   in
   restore (); r
@@ -136,13 +228,13 @@ let expand_quotation gloc expander shift name str =
 let parse_quotation_result entry loc shift name str =
   let cs = Stream.of_string str in
   try Grammar.Entry.parse entry cs with
-    Ploc.Exc (iloc, Qerror (_, Expanding, exc)) ->
+    Ploc.Exc (iloc, Qerror (_, _, Expanding, exc)) ->
       let ctx = ParsingResult (iloc, str) in
-      let exc1 = Qerror (name, ctx, exc) in Ploc.raise loc exc1
-  | Ploc.Exc (_, (Qerror (_, _, _) as exc)) -> Ploc.raise loc exc
+      let exc1 = Qerror (name, str, ctx, exc) in Ploc.raise loc exc1
+  | Ploc.Exc (_, (Qerror (_, _, _, _) as exc)) -> Ploc.raise loc exc
   | Ploc.Exc (iloc, exc) ->
       let ctx = ParsingResult (iloc, str) in
-      let exc1 = Qerror (name, ctx, exc) in Ploc.raise loc exc1
+      let exc1 = Qerror (name, str, ctx, exc) in Ploc.raise loc exc1
 ;;
 
 let handle_quotation loc proj proj2 in_expr entry reloc (name, str) =
@@ -164,7 +256,7 @@ let handle_quotation loc proj proj2 in_expr entry reloc (name, str) =
   let expander =
     try Quotation.find name with
       exc ->
-        let exc1 = Qerror (name, Finding, exc) in
+        let exc1 = Qerror (name, str, Finding, exc) in
         raise (Ploc.Exc (Ploc.sub loc 0 shift, exc1))
   in
   let ast =
@@ -188,17 +280,25 @@ let handle_quotation loc proj proj2 in_expr entry reloc (name, str) =
 
 let expr_eoi = Grammar.Entry.create gram "expr_eoi";;
 let patt_eoi = Grammar.Entry.create gram "patt_eoi";;
-Grammar.extend
-  [Grammar.Entry.obj (expr_eoi : 'expr_eoi Grammar.Entry.e), None,
-   [None, None,
-    [[Gramext.Snterm (Grammar.Entry.obj (expr : 'expr Grammar.Entry.e));
-      Gramext.Stoken ("EOI", "")],
-     Gramext.action (fun _ (x : 'expr) (loc : Ploc.t) -> (x : 'expr_eoi))]];
-   Grammar.Entry.obj (patt_eoi : 'patt_eoi Grammar.Entry.e), None,
-   [None, None,
-    [[Gramext.Snterm (Grammar.Entry.obj (patt : 'patt Grammar.Entry.e));
-      Gramext.Stoken ("EOI", "")],
-     Gramext.action (fun _ (x : 'patt) (loc : Ploc.t) -> (x : 'patt_eoi))]]];;
+Grammar.safe_extend
+  [Grammar.extension (expr_eoi : 'expr_eoi Grammar.Entry.e) None
+     [None, None,
+      [Grammar.production
+         (Grammar.r_next
+            (Grammar.r_next Grammar.r_stop
+               (Grammar.s_nterm (expr : 'expr Grammar.Entry.e)))
+            (Grammar.s_token ("EOI", "")),
+          "194fe98d",
+          (fun _ (x : 'expr) (loc : Ploc.t) -> (x : 'expr_eoi)))]];
+   Grammar.extension (patt_eoi : 'patt_eoi Grammar.Entry.e) None
+     [None, None,
+      [Grammar.production
+         (Grammar.r_next
+            (Grammar.r_next Grammar.r_stop
+               (Grammar.s_nterm (patt : 'patt Grammar.Entry.e)))
+            (Grammar.s_token ("EOI", "")),
+          "194fe98d",
+          (fun _ (x : 'patt) (loc : Ploc.t) -> (x : 'patt_eoi)))]]];;
 
 let handle_expr_quotation loc x =
   handle_quotation loc fst fst true expr_eoi Reloc.expr x
@@ -232,16 +332,16 @@ let string_of_loc fname line bp ep =
   | _ -> sprintf "File \"%s\", line %d, characters %d-%d:\n" fname line bp ep
 ;;
 
-let report_quotation_error name ctx =
+let pp_report_quotation_error pps name str ctx =
   let name = if name = "" then !(Quotation.default) else name in
-  Format.print_flush ();
-  Format.open_hovbox 2;
-  eprintf "While %s \"%s\":"
+  Format.pp_print_flush pps ();
+  Format.pp_open_hovbox pps 2;
+  eprintf "While %s \"%s\" for string \"%s\":"
     (match ctx with
        Finding -> "finding quotation"
      | Expanding -> "expanding quotation"
      | ParsingResult (_, _) -> "parsing result of quotation")
-    name;
+    name str;
   match ctx with
     ParsingResult (loc, str) ->
       begin match !quotation_dump_file with
@@ -272,9 +372,10 @@ let report_quotation_error name ctx =
   | _ -> eprintf "\n"; flush stderr
 ;;
 
-let print_format str =
-  let rec flush ini cnt =
-    if cnt > ini then Format.print_string (String.sub str ini (cnt - ini))
+let pp_print_format pps str =
+  let flush ini cnt =
+    if cnt > ini then
+      Format.pp_print_string pps (String.sub str ini (cnt - ini))
   in
   let rec loop ini cnt =
     if cnt == String.length str then flush ini cnt
@@ -282,83 +383,85 @@ let print_format str =
       match str.[cnt] with
         '\n' ->
           flush ini cnt;
-          Format.close_box ();
-          Format.force_newline ();
-          Format.open_box 2;
+          Format.pp_close_box pps ();
+          Format.pp_force_newline pps ();
+          Format.pp_open_box pps 2;
+          loop (cnt + 1) (cnt + 1)
+      | ' ' ->
+          flush ini cnt;
+          Format.pp_print_space pps ();
           loop (cnt + 1) (cnt + 1)
-      | ' ' -> flush ini cnt; Format.print_space (); loop (cnt + 1) (cnt + 1)
       | _ -> loop ini (cnt + 1)
   in
-  Format.open_box 2; loop 0 0; Format.close_box ()
+  Format.pp_open_box pps 2; loop 0 0; Format.pp_close_box pps ()
 ;;
 
-let print_file_failed file line char =
-  Format.print_string ", file \"";
-  Format.print_string file;
-  Format.print_string "\", line ";
-  Format.print_int line;
-  Format.print_string ", char ";
-  Format.print_int char
+let pp_print_file_failed pps file line char =
+  Format.pp_print_string pps ", file \"";
+  Format.pp_print_string pps file;
+  Format.pp_print_string pps "\", line ";
+  Format.pp_print_int pps line;
+  Format.pp_print_string pps ", char ";
+  Format.pp_print_int pps char
 ;;
 
-let print_exn =
+let pp_print_exn pps =
   function
-    Out_of_memory -> Format.print_string "Out of memory\n"
+    Out_of_memory -> Format.pp_print_string pps "Out of memory\n"
   | Assert_failure (file, line, char) ->
-      Format.print_string "Assertion failed"; print_file_failed file line char
+      Format.pp_print_string pps "Assertion failed";
+      pp_print_file_failed pps file line char
   | Match_failure (file, line, char) ->
-      Format.print_string "Pattern matching failed";
-      print_file_failed file line char
+      Format.pp_print_string pps "Pattern matching failed";
+      pp_print_file_failed pps file line char
   | Stream.Error str ->
-      if str = "" then Format.print_string "Parse error"
-      else print_format ("Parse error: " ^ str)
-  | Stream.Failure -> Format.print_string "Parse failure"
+      if str = "" then Format.pp_print_string pps "Parse error"
+      else pp_print_format pps ("Parse error: " ^ str)
+  | Stream.Failure -> Format.pp_print_string pps "Parse failure"
   | Plexing.Error str ->
-      Format.print_string "Lexing error";
+      Format.pp_print_string pps "Lexing error";
       if str <> "" then
-        begin Format.print_string ": "; Format.print_string str end
-      else Format.print_string "."
-  | Failure str -> Format.print_string "Failure: "; Format.print_string str
-  | Invalid_argument str ->
-      Format.print_string "Invalid argument: "; Format.print_string str
-  | Sys_error msg ->
-      Format.print_string "I/O error: "; Format.print_string msg
-  | x ->
-      Format.print_string "Uncaught exception: ";
-      Format.print_string
-        (Obj.magic (Obj.field (Obj.field (Obj.repr x) 0) 0));
-      if Obj.size (Obj.repr x) > 1 then
         begin
-          Format.print_string " (";
-          for i = 1 to Obj.size (Obj.repr x) - 1 do
-            if i > 1 then Format.print_string ", ";
-            let arg = Obj.field (Obj.repr x) i in
-            if not (Obj.is_block arg) then
-              Format.print_int (Obj.magic arg : int)
-            else if Obj.tag arg = Obj.tag (Obj.repr "a") then
-              begin
-                Format.print_char '"';
-                Format.print_string (Obj.magic arg : string);
-                Format.print_char '"'
-              end
-            else Format.print_char '_'
-          done;
-          Format.print_char ')'
+          Format.pp_print_string pps ": ";
+          Format.pp_print_string pps str
         end
+      else Format.pp_print_string pps "."
+  | Failure str ->
+      Format.pp_print_string pps "Failure: "; Format.pp_print_string pps str
+  | Invalid_argument str ->
+      Format.pp_print_string pps "Invalid argument: ";
+      Format.pp_print_string pps str
+  | Sys_error msg ->
+      Format.pp_print_string pps "I/O error: "; Format.pp_print_string pps msg
+  | x ->
+      Format.pp_print_string pps "Uncaught exception: ";
+      Format.pp_print_string pps (Printexc.to_string x)
 ;;
 
-let report_error exn =
+let pp_report_error pps exn =
   match exn with
-    Qerror (name, Finding, Not_found) ->
+    Qerror (name, str, Finding, Not_found) ->
       let name = if name = "" then !(Quotation.default) else name in
-      Format.print_flush ();
-      Format.open_hovbox 2;
-      Format.printf "Unbound quotation: \"%s\"" name;
-      Format.close_box ()
-  | Qerror (name, ctx, exn) -> report_quotation_error name ctx; print_exn exn
-  | e -> print_exn exn
+      Format.pp_print_flush pps ();
+      Format.pp_open_hovbox pps 2;
+      Format.fprintf pps "Unbound quotation: \"%s\" for string \"%s\"" name
+        str;
+      Format.pp_close_box pps ()
+  | Qerror (name, str, ctx, exn) ->
+      pp_report_quotation_error pps name str ctx; pp_print_exn pps exn
+  | e -> pp_print_exn pps exn
 ;;
 
+let report_error exn = pp_report_error Format.std_formatter exn;;
+
+Printexc.register_printer
+  (function
+     Qerror (_, _, _, _) as exn ->
+       let b = Buffer.create 23 in
+       let pps = Format.formatter_of_buffer b in
+       Format.fprintf pps "%a%!" pp_report_error exn; Some (Buffer.contents b)
+   | _ -> None);;
+
 let no_constructors_arity = Prtools.no_constructors_arity;;
 
 let arg_spec_list_ref = ref [];;
@@ -373,11 +476,12 @@ let undef x = ref (fun _ -> failwith x);;
 let print_interf = undef "no printer";;
 let print_implem = undef "no printer";;
 
-let pr_expr = Eprinter.make "expr";;
+let pr_expr = Eprinter.make ~fail:Pp_debug.Pp_MLast.show_expr "expr";;
 let pr_patt = Eprinter.make "patt";;
 let pr_ctyp = Eprinter.make "type";;
 let pr_str_item = Eprinter.make "str_item";;
 let pr_sig_item = Eprinter.make "sig_item";;
+let pr_longident = Eprinter.make "longident";;
 let pr_module_expr = Eprinter.make "module_expr";;
 let pr_module_type = Eprinter.make "module_type";;
 let pr_class_sig_item = Eprinter.make "class_sig_item";;
@@ -388,7 +492,7 @@ let pr_expr_fun_args = ref Extfun.empty;;
 
 let flag_comments_in_phrases = ref true;;
 let flag_equilibrate_cases = ref false;;
-let flag_compatible_old_versions_of_ocaml = ref false;;
+let flag_expand_letop_syntax = ref false;;
 
 let inter_phrases = ref None;;
 
@@ -454,6 +558,8 @@ let greek_ascii_equiv s =
 let strict_mode = ref false;;
 
 let unvala x = x;;
+let vala_it f x = ignore (f x);;
+
 let vala_map f x = f x;;
 let vala_mapa f g x = f x;;
 
