@@ -1,6 +1,6 @@
 (* camlp5r pa_macro.cmo *)
 (* odyl_main.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
+(* Copyright (c) INRIA 2007-2017 *)
 
 value go = ref (fun () -> ());
 value name = ref "odyl";
@@ -50,7 +50,7 @@ value loadfile file =
   ELSE do {
     if not initialized.val then do {
       IFDEF OPT THEN ()
-      ELSE do { Dynlink.init (); Dynlink.allow_unsafe_modules True }
+      ELSE do { Dynlink.allow_unsafe_modules True }
       END;
       initialized.val := True
     }
@@ -64,7 +64,12 @@ value loadfile file =
       [ Not_found -> raise (Error file "file not found in path") ]
     in
     try Dynlink.loadfile fname with
-    [ Dynlink.Error e -> raise (Error fname (Dynlink.error_message e)) ]
+    | IFDEF OCAML_VERSION >= OCAML_4_08 AND NOT BOOTSTRAPPING_SOURCES THEN
+      Dynlink.Error (Module_already_loaded _) ->
+        () (* ignore for compatibility *)
+      END
+    | Dynlink.Error e -> raise (Error fname (Dynlink.error_message e))
+    end
   }
   END
 ;
