@@ -1,20 +1,32 @@
 (* camlp5r pa_macro.cmo *)
 (* rprint.ml,v *)
-(* Copyright (c) INRIA 2007-2014 *)
-
-IFDEF OCAML_VERSION >= OCAML_3_03 THEN
+(* Copyright (c) INRIA 2007-2017 *)
 
 open Format;
 open Outcometree;
+open Pp_debug ;
+
+value string_lowercase =
+  IFDEF OCAML_VERSION < OCAML_4_03_0 THEN String.lowercase
+  ELSE String.lowercase_ascii END
+;
 
 exception Ellipsis;
 value cautious f ppf arg =
   try f ppf arg with [ Ellipsis -> fprintf ppf "..." ]
 ;
 
+value print_out_name ppf s =
+  IFDEF OCAML_VERSION < OCAML_4_08_0 THEN
+    fprintf ppf "%s" s
+  ELSE
+    fprintf ppf "%s" s.printed_name
+  END
+;
+
 value rec print_ident ppf =
   fun
-  [ Oide_ident s -> fprintf ppf "%s" s
+  [ Oide_ident s -> print_out_name ppf s
   | Oide_dot id s -> fprintf ppf "%a.%s" print_ident id s
   | Oide_apply id1 id2 ->
       fprintf ppf "%a(%a)" print_ident id1 print_ident id2 ]
@@ -31,6 +43,16 @@ value value_ident ppf name =
     | _ -> fprintf ppf "( %s )" name ]
 ;
 
+(* copied from Ocaml 4.10.0 sources *)
+value tyvar ppf s =
+  if String.length s >= 2 && s.[1] = '\'' then
+    (* without the space, this would be parsed as
+       a character literal *)
+    Format.fprintf ppf "' %s" s
+  else
+    Format.fprintf ppf "'%s" s
+;
+
 (* Values *)
 
 value print_out_value ppf tree =
@@ -45,26 +67,36 @@ value print_out_value ppf tree =
   and print_simple_tree ppf =
     fun
     [ Oval_int i -> fprintf ppf "%i" i
-    | IFDEF OCAML_VERSION >= OCAML_3_07 THEN
-        Oval_int32 i -> fprintf ppf "%lil" i
-      END
-    | IFDEF OCAML_VERSION >= OCAML_3_07 THEN
-        Oval_int64 i -> fprintf ppf "%LiL" i
-      END
-    | IFDEF OCAML_VERSION >= OCAML_3_07 THEN
-        Oval_nativeint i -> fprintf ppf "%nin" i
-      END
+    | Oval_int32 i -> fprintf ppf "%lil" i
+    | Oval_int64 i -> fprintf ppf "%LiL" i
+    | Oval_nativeint i -> fprintf ppf "%nin" i
     | Oval_float f -> fprintf ppf "%.12g" f
     | Oval_char c -> fprintf ppf "'%s'" (Char.escaped c)
-    | Oval_string s ->
+    | IFDEF OCAML_VERSION < OCAML_4_06 THEN
+      Oval_string s ->
         try fprintf ppf "\"%s\"" (String.escaped s) with
-        [ Invalid_argument "String.create" -> fprintf ppf "<huge string>" ]
+        [ Invalid_argument _ -> fprintf ppf "<huge string>" ]
+      ELSE
+      Oval_string s _ _ ->
+        try fprintf ppf "\"%s\"" (String.escaped s) with
+        [ Invalid_argument _ -> fprintf ppf "<huge string>" ]
+      END
     | Oval_list tl ->
         fprintf ppf "@[<1>[%a]@]" (print_tree_list print_tree ";") tl
     | Oval_array tl ->
         fprintf ppf "@[<2>[|%a|]@]" (print_tree_list print_tree ";") tl
-    | Oval_constr (Oide_ident "true") [] -> fprintf ppf "True"
-    | Oval_constr (Oide_ident "false") [] -> fprintf ppf "False"
+    | IFDEF OCAML_VERSION < OCAML_4_08_0 THEN
+      Oval_constr (Oide_ident "true") [] -> fprintf ppf "True"
+      ELSE
+      Oval_constr (Oide_ident {printed_name = "true"}) [] ->
+        fprintf ppf "True"
+      END
+    | IFDEF OCAML_VERSION < OCAML_4_08_0 THEN
+      Oval_constr (Oide_ident "false") [] -> fprintf ppf "False"
+      ELSE
+      Oval_constr (Oide_ident {printed_name = "false"}) [] ->
+        fprintf ppf "False"
+      END
     | Oval_constr name [] -> print_ident ppf name
     | Oval_variant name None -> fprintf ppf "`%s" name
     | Oval_stuff s -> fprintf ppf "%s" s
@@ -81,8 +113,14 @@ value print_out_value ppf tree =
     | [(name, tree) :: fields] ->
         let name =
           match name with
-          [ Oide_ident "contents" -> Oide_ident "val"
-          | x -> x ]
+          | IFDEF OCAML_VERSION < OCAML_4_08_0 THEN
+            Oide_ident "contents" -> Oide_ident "val"
+            ELSE
+            Oide_ident {printed_name = "contents"} ->
+              Oide_ident {printed_name = "val"}
+            END
+          | x -> x
+          end
         in
         do {
           if not first then fprintf ppf ";@ " else ();
@@ -130,7 +168,7 @@ value utf8 =
   let slen = String.length s in
   let ulen = String.length utf8_str in
   slen >= ulen &&
-  String.lowercase (String.sub s (slen - ulen) ulen) = utf8_str
+  string_lowercase (String.sub s (slen - ulen) ulen) = utf8_str
 ;
 
 (* Type variables in Greek *)
@@ -159,9 +197,15 @@ value try_greek s = do {
 
 value rec print_out_type ppf =
   fun
-  [ Otyp_alias ty s ->
+  [ IFDEF OCAML_VERSION < OCAML_5_1_0 THEN
+    Otyp_alias ty s ->
       let (q, s) = try_greek s in
       fprintf ppf "@[%a as %s%s@]" print_out_type ty q s
+    ELSE
+    Otyp_alias {non_gen=non_gen; aliased=ty; \alias =s} ->
+      let (q, s) = try_greek s in
+      fprintf ppf "@[%a as %s%s%s@]" print_out_type ty (if non_gen then "_" else "") q s
+    END
   | ty -> print_out_type_1 ppf ty ]
 and print_out_type_1 ppf =
   fun
@@ -175,55 +219,12 @@ and print_out_type_2 ppf =
       fprintf ppf "@[%a@;<1 2>%a@]" print_ident id
         (print_typlist print_simple_out_type "") tyl
   | ty -> print_simple_out_type ppf ty ]
-and print_simple_out_type ppf =
-  fun
-  [ Otyp_var ng s ->
-      let (q, s) = try_greek s in
-      fprintf ppf "%s%s%s" q (if ng then "_" else "") s
-  | Otyp_constr id [] -> fprintf ppf "@[%a@]" print_ident id
-  | Otyp_tuple tyl ->
-      fprintf ppf "@[<1>(%a)@]" (print_typlist print_out_type " *") tyl
-  | Otyp_stuff s -> fprintf ppf "%s" s
-  | Otyp_variant non_gen poly_variants closed tags ->
-      let print_present ppf =
-        fun
-        [ None | Some [] -> ()
-        | Some l -> fprintf ppf "@;<1 -2>> @[<hov>%a@]" pr_present l ]
-      in
-      let print_fields ppf =
-        fun
-        [ Ovar_fields fields ->
-            print_list print_poly_variant
-              (fun ppf -> fprintf ppf "@;<1 -2>| ") ppf fields
-        | Ovar_name id tyl ->
-            fprintf ppf "@[%a%a@]" print_typargs tyl print_ident id ]
-      in
-      fprintf ppf "%s[%s@[<hv>@[<hv>%a@]%a ]@]" (if non_gen then "_" else "")
-        (if closed then if tags = None then "= " else "< "
-         else if tags = None then "> "
-         else "? ")
-        print_fields poly_variants
-        print_present tags
-  | Otyp_object fields rest ->
-      fprintf ppf "@[<2>< %a >@]" (print_fields rest) fields
-  | Otyp_class ng id tyl ->
-      fprintf ppf "@[%a%s#%a@]" print_typargs tyl (if ng then "_" else "")
-        print_ident id
-  | Otyp_manifest ty1 ty2 ->
-      fprintf ppf "@[<2>%a ==@ %a@]" print_out_type ty1 print_out_type ty2
-  | Otyp_abstract -> fprintf ppf "'abstract"
-  | IFDEF OCAML_VERSION >= OCAML_4_02_0 THEN
-    Otyp_open -> fprintf ppf "open"
-    END
-  | Otyp_alias _ _ | Otyp_arrow _ _ _ | Otyp_constr _ [_ :: _] as ty ->
-      fprintf ppf "@[<1>(%a)@]" print_out_type ty
-  | IFDEF OCAML_VERSION >= OCAML_3_05 THEN
-      Otyp_poly _ _ as ty ->
-        fprintf ppf "@[<1>(%a)@]" print_out_type ty
-    END
-  | IFDEF OCAML_VERSION >= OCAML_3_12_0 THEN
-      Otyp_module p n tyl -> do {
-        fprintf ppf "@[<1>(module %s" p;
+and print_module_out_type ppf p n tyl = do {
+        IFDEF OCAML_VERSION < OCAML_4_08_0 THEN
+          fprintf ppf "@[<1>(module %s" p
+        ELSE
+          fprintf ppf "@[<1>(module %a" print_ident p
+        END;
         let first = ref True in
         List.iter2
           (fun s t ->
@@ -234,36 +235,119 @@ and print_simple_out_type ppf =
           n tyl;
         fprintf ppf ")@]"
       }
+and print_simple_out_type ppf =
+  fun
+  [ Otyp_var ng s ->
+      let (q, s) = try_greek s in
+      fprintf ppf "%s%s%s" q (if ng then "_" else "") s
+  | Otyp_constr id [] -> fprintf ppf "@[%a@]" print_ident id
+  | Otyp_tuple tyl ->
+      fprintf ppf "@[<1>(%a)@]" (print_typlist print_out_type " *") tyl
+  | Otyp_stuff s -> fprintf ppf "%s" s
+  | IFDEF OCAML_VERSION < OCAML_5_1_0 THEN
+    Otyp_variant non_gen poly_variants closed tags ->
+      let print_present ppf =
+        fun
+        [ None | Some [] -> ()
+        | Some l -> fprintf ppf "@;<1 -2>> @[<hov>%a@]" pr_present l ]
+      in
+      let print_fields ppf =
+        fun
+        | Ovar_fields fields ->
+            print_list print_poly_variant
+              (fun ppf -> fprintf ppf "@;<1 -2>| ") ppf fields
+        | IFDEF OCAML_VERSION < OCAML_4_05_0 THEN
+          Ovar_name id tyl ->
+            fprintf ppf "@[%a%a@]" print_typargs tyl print_ident id
+          ELSE
+          Ovar_typ ty ->
+            fprintf ppf "%a@ " print_simple_out_type ty
+          END
+        end
+      in
+      fprintf ppf "%s[%s@[<hv>@[<hv>%a@]%a ]@]" (if non_gen then "_" else "")
+        (if closed then if tags = None then "= " else "< "
+         else if tags = None then "> "
+         else "? ")
+        print_fields poly_variants
+        print_present tags
+    ELSE
+    Otyp_variant poly_variants closed tags ->
+      let print_present ppf =
+        fun
+        [ None | Some [] -> ()
+        | Some l -> fprintf ppf "@;<1 -2>> @[<hov>%a@]" pr_present l ]
+      in
+      let print_fields ppf =
+        fun
+        | Ovar_fields fields ->
+            print_list print_poly_variant
+              (fun ppf -> fprintf ppf "@;<1 -2>| ") ppf fields
+        | IFDEF OCAML_VERSION < OCAML_4_05_0 THEN
+          Ovar_name id tyl ->
+            fprintf ppf "@[%a%a@]" print_typargs tyl print_ident id
+          ELSE
+          Ovar_typ ty ->
+            fprintf ppf "%a@ " print_simple_out_type ty
+          END
+        end
+      in
+      fprintf ppf "[%s@[<hv>@[<hv>%a@]%a ]@]"
+        (if closed then if tags = None then "= " else "< "
+         else if tags = None then "> "
+         else "? ")
+        print_fields poly_variants
+        print_present tags
     END
-  | IFDEF OCAML_VERSION >= OCAML_3_13_0 AND NOT JOCAML THEN
-      Otyp_sum constrs ->
+  | IFDEF OCAML_VERSION < OCAML_5_1_0 THEN
+    Otyp_object fields rest ->
+      fprintf ppf "@[<2>< %a >@]" (print_fields rest) fields
+    ELSE
+    Otyp_object {fields=fields; open_row=open_row} ->
+      fprintf ppf "@[<2>< %a >@]" (print_fields open_row) fields
+    END
+  | IFDEF OCAML_VERSION < OCAML_5_1_0 THEN
+    Otyp_class ng id tyl ->
+      fprintf ppf "@[%a%s#%a@]" print_typargs tyl (if ng then "_" else "")
+        print_ident id
+    ELSE
+    Otyp_class id tyl ->
+      fprintf ppf "@[%a#%a@]" print_typargs tyl
+        print_ident id
+    END
+  | Otyp_manifest ty1 ty2 ->
+      fprintf ppf "@[<2>%a ==@ %a@]" print_out_type ty1 print_out_type ty2
+  | Otyp_abstract -> fprintf ppf "'abstract"
+  | IFDEF OCAML_VERSION >= OCAML_4_05_0 THEN
+    Otyp_open -> fprintf ppf "open"
+    END
+  | IFDEF OCAML_VERSION < OCAML_5_1_0 THEN
+    Otyp_alias _ _ | Otyp_arrow _ _ _ | Otyp_constr _ [_ :: _] as ty ->
+      fprintf ppf "@[<1>(%a)@]" print_out_type ty    
+    ELSE
+    Otyp_alias _ | Otyp_arrow _ _ _ | Otyp_constr _ [_ :: _] as ty ->
+      fprintf ppf "@[<1>(%a)@]" print_out_type ty
+    END
+  | Otyp_poly _ _ as ty ->
+        fprintf ppf "@[<1>(%a)@]" print_out_type ty
+  | IFDEF OCAML_VERSION < OCAML_4_13_0 THEN
+    Otyp_module p n tyl ->
+      print_module_out_type ppf p n tyl
+    ELSE
+    Otyp_module p l ->
+      print_module_out_type ppf p (List.map fst l) (List.map snd l)
+    END
+  | Otyp_sum constrs ->
         fprintf ppf "@[<hv>[ %a ]@]"
           (print_list print_out_constr_gadt_opt
              (fun ppf -> fprintf ppf "@ | "))
           constrs
-    ELSIFDEF OCAML_VERSION > OCAML_3_06 AND OCAML_VERSION <= OCAML_3_08_4 THEN
-      Otyp_sum constrs _ ->
-        fprintf ppf "@[<hv>[ %a ]@]"
-          (print_list print_out_constr (fun ppf -> fprintf ppf "@ | "))
-          constrs
-    ELSE
-      Otyp_sum constrs ->
-        fprintf ppf "@[<hv>[ %a ]@]"
-          (print_list print_out_constr (fun ppf -> fprintf ppf "@ | "))
-          constrs
-    END
-  | IFDEF OCAML_VERSION > OCAML_3_06 AND OCAML_VERSION <= OCAML_3_08_4 THEN
-      Otyp_record lbls _ ->
+    
+  | Otyp_record lbls ->
         fprintf ppf "@[<hv 2>{ %a }@]"
           (print_list print_out_label (fun ppf -> fprintf ppf ";@ ")) lbls
-    ELSE
-      Otyp_record lbls ->
-        fprintf ppf "@[<hv 2>{ %a }@]"
-          (print_list print_out_label (fun ppf -> fprintf ppf ";@ ")) lbls
-    END
-  | IFDEF JOCAML THEN
-      Otyp_proc ->
-        failwith "Rprint: Otyp_proc not impl"
+  | IFDEF OCAML_VERSION >= OCAML_4_05_0 THEN
+      Otyp_attribute _ _ -> ()
     END ]
 and print_out_constr ppf (name, tyl) =
   match tyl with
@@ -271,28 +355,50 @@ and print_out_constr ppf (name, tyl) =
   | _ ->
       fprintf ppf "@[<2>%s of@ %a@]" name
         (print_typlist print_out_type " and") tyl ]
-and print_out_constr_gadt_opt ppf (name, tyl, rto) =
+and print_out_constr_gadt_opt ppf = fun [
+  IFDEF OCAML_VERSION < OCAML_4_14_0 THEN
+ (name, tyl, rto) ->
   match rto with
   [ Some rt ->
       let t = List.fold_right (fun t1 t2 -> Otyp_arrow "" t1 t2) tyl rt in
       fprintf ppf "%s : %a" name print_out_type t
   | None -> print_out_constr ppf (name, tyl) ]
+  ELSE
+ {ocstr_name=name; ocstr_args=tyl; ocstr_return_type=rto} ->
+  match rto with
+  [ Some rt ->
+      let t = List.fold_right (fun t1 t2 -> Otyp_arrow "" t1 t2) tyl rt in
+      fprintf ppf "%s : %a" name print_out_type t
+  | None -> print_out_constr ppf (name, tyl) ]
+  END
+]
 and print_out_label ppf (name, mut, arg) =
   fprintf ppf "@[<2>%s :@ %s%a@]" name (if mut then "mutable " else "")
     print_out_type arg
 and print_fields rest ppf =
   fun
   [ [] ->
+    IFDEF OCAML_VERSION < OCAML_5_1_0 THEN
       match rest with
       [ Some non_gen -> fprintf ppf "%s.." (if non_gen then "_" else "")
       | None -> () ]
+    ELSE
+      fprintf ppf "%s.." (if rest then "_" else "")
+    END
+
   | [(s, t)] ->
       do {
         fprintf ppf "%s : %a" s print_out_type t;
-        match rest with
-        [ Some _ -> fprintf ppf ";@ "
-        | None -> () ];
-        print_fields rest ppf []
+        IFDEF OCAML_VERSION < OCAML_5_1_0 THEN
+          do {
+            match rest with
+            [ Some _ -> fprintf ppf ";@ "
+            | None -> () ] ;
+            print_fields rest ppf []
+          }
+        ELSE
+          fprintf ppf ";@ "
+        END
       }
   | [(s, t) :: l] ->
       fprintf ppf "%s : %a;@ %a" s print_out_type t (print_fields rest) l ]
@@ -342,15 +448,9 @@ value rec print_out_class_type ppf =
               (print_typlist print_out_type ",") tyl ]
       in
       fprintf ppf "@[%a%a@]" pr_tyl tyl print_ident id
-  | IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
-    Octy_fun lab ty cty ->
+  | Octy_arrow lab ty cty ->
       fprintf ppf "@[%s[ %a ] ->@ %a@]" (if lab <> "" then lab ^ ":" else "")
         print_out_type ty print_out_class_type cty
-    ELSE
-    Octy_arrow lab ty cty ->
-      fprintf ppf "@[%s[ %a ] ->@ %a@]" (if lab <> "" then lab ^ ":" else "")
-        print_out_type ty print_out_class_type cty
-    END
   | Octy_signature self_ty csil ->
       let pr_param ppf =
         fun
@@ -370,16 +470,8 @@ and print_out_class_sig_item ppf =
         (if priv then "private " else "") (if virt then "virtual " else "")
         name print_out_type ty
   | x ->
-      IFDEF OCAML_VERSION >= OCAML_3_10 THEN
         failwith "Rprint.print_out_class_sig_item: not impl"
-      ELSE
-        match x with
-        [ Ocsg_value name mut ty ->
-            fprintf ppf "@[<2>value %s%s :@ %a;@]"
-               (if mut then "mutable " else "") name
-               print_out_type ty
-        | _ -> assert False ]
-      END ]
+       ]
 ;
 
 value rec print_out_module_type ppf =
@@ -387,11 +479,8 @@ value rec print_out_module_type ppf =
   [ Omty_ident id -> fprintf ppf "%a" print_ident id
   | Omty_signature sg ->
       fprintf ppf "@[<hv 2>sig@ %a@;<1 -2>end@]" print_out_signature sg
-  | Omty_functor name mty_arg mty_res ->
-      IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
-        fprintf ppf "@[<2>functor@ (%s : %a) ->@ %a@]" name
-          print_out_module_type mty_arg print_out_module_type mty_res
-      ELSE
+  | IFDEF OCAML_VERSION < OCAML_4_10_0 THEN
+    Omty_functor name mty_arg mty_res ->
         match mty_arg with
         [ Some mty_arg ->
             fprintf ppf "@[<2>functor@ (%s : %a) ->@ %a@]" name
@@ -399,9 +488,21 @@ value rec print_out_module_type ppf =
         | None ->
             fprintf ppf "@[<2>functor@ (%s) ->@ %a@]" name
               print_out_module_type mty_res ]
+    ELSE
+    Omty_functor mty_arg mty_res ->
+       match mty_arg with
+        [ Some (None, mty_arg) ->
+            fprintf ppf "@[<2>%a ->@ %a@]"
+              print_out_module_type mty_arg print_out_module_type mty_res
+        | Some (Some name, mty_arg) ->
+            fprintf ppf "@[<2>functor@ (%s : %a) ->@ %a@]" name
+              print_out_module_type mty_arg print_out_module_type mty_res
+        | None ->
+            fprintf ppf "@[<2>functor@ () ->@ %a@]"
+              print_out_module_type mty_res ]
       END
   | Omty_abstract -> ()
-  | IFDEF OCAML_VERSION >= OCAML_4_02_0 THEN
+  | IFDEF OCAML_VERSION >= OCAML_4_05_0 THEN
     Omty_alias oi -> fprintf ppf "<rprint.ml: Omty_alias not impl>"
     END ]
 and print_out_signature ppf =
@@ -413,50 +514,76 @@ and print_out_signature ppf =
         print_out_signature items ]
 and print_out_sig_item ppf =
   fun
-  [ IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
-    Osig_exception id tyl ->
-      fprintf ppf "@[<2>exception %a@]" print_out_constr (id, tyl)
-    END
+  [ Osig_typext ext _es ->
+      match _es with [
+        Oext_exception ->
+          fprintf ppf "@[<2>exception %a@]"
+            print_out_constr_gadt_opt
+            (IFDEF OCAML_VERSION < OCAML_4_14_0 THEN
+            (ext.oext_name, ext.oext_args, ext.oext_ret_type)
+            ELSE
+            {ocstr_name=ext.oext_name; ocstr_args=ext.oext_args; ocstr_return_type=ext.oext_ret_type}
+            END)
+      | _ ->
+        let print_out_extension_constructor ppf ext =
+          let pr_var = tyvar in
+          let print_type_parameter ppf s =
+            if s = "_" then fprintf ppf "_" else pr_var ppf s in
+          let print_extended_type ppf =
+            match ext.oext_type_params with [
+              [] -> fprintf ppf "%s" ext.oext_type_name
+            | [ty_param] ->
+              fprintf ppf "@[%a@ %s@]"
+                print_type_parameter
+                ty_param
+                ext.oext_type_name
+            | _ ->
+              fprintf ppf "@[(@[%a)@]@ %s@]"
+                (print_list print_type_parameter (fun ppf -> fprintf ppf ",@ "))
+                ext.oext_type_params
+                ext.oext_type_name
+            ]
+          in
+          fprintf ppf "@[<hv 2>type %t +=%s@;<1 2>%a@]"
+            print_extended_type
+            (if ext.oext_private = Asttypes.Private then " private" else "")
+            print_out_constr_gadt_opt
+            (IFDEF OCAML_VERSION < OCAML_4_14_0 THEN
+            (ext.oext_name, ext.oext_args, ext.oext_ret_type)
+            ELSE
+            {ocstr_name=ext.oext_name; ocstr_args=ext.oext_args; ocstr_return_type=ext.oext_ret_type}
+            END)
+        in
+        print_out_extension_constructor ppf ext
+      ]
   | Osig_modtype name Omty_abstract ->
       fprintf ppf "@[<2>module type %s = 'a@]" name
   | Osig_modtype name mty ->
       fprintf ppf "@[<2>module type %s =@ %a@]" name print_out_module_type mty
-  | IFDEF OCAML_VERSION <= OCAML_3_07 THEN
-      Osig_module name mty ->
-        fprintf ppf "@[<2>module %s :@ %a@]" name print_out_module_type mty
-    ELSE
-      Osig_module name mty _ ->
+  | Osig_module name mty _ ->
         fprintf ppf "@[<2>module %s :@ %a@]" name
           print_out_module_type mty
-    END
-  | IFDEF OCAML_VERSION <= OCAML_3_07 THEN
-      Osig_type tdl -> do {
-        print_out_type_decl "type" ppf (List.hd tdl);
-        List.iter (print_out_type_decl "and" ppf) (List.tl tdl);
-      }
-    ELSE
-      Osig_type td rs ->
+  | Osig_type td rs ->
         print_out_type_decl (if rs = Orec_next then "and" else "type" )
           ppf td
-    END
-  | Osig_value name ty prims ->
-      let kwd = if prims = [] then "value" else "external" in
-      let pr_prims ppf =
-        fun
-        [ [] -> ()
-        | [s :: sl] ->
-            do {
-              fprintf ppf "@ = \"%s\"" s;
-              List.iter (fun s -> fprintf ppf "@ \"%s\"" s) sl
-            } ]
-      in
-      fprintf ppf "@[<2>%s %a :@ %a%a@]" kwd value_ident name
-        print_out_type ty pr_prims prims
+  |   Osig_value ovd ->
+        let name = ovd.oval_name in
+        let ty = ovd.oval_type in
+        let prims = ovd.oval_prims in
+        let kwd = if prims = [] then "value" else "external" in
+        let pr_prims ppf =
+          fun
+          [ [] -> ()
+          | [s :: sl] ->
+              do {
+                fprintf ppf "@ = \"%s\"" s;
+                List.iter (fun s -> fprintf ppf "@ \"%s\"" s) sl
+              } ]
+        in
+        fprintf ppf "@[<2>%s %a :@ %a%a@]" kwd value_ident name
+          print_out_type ty pr_prims prims
   | x ->
-      IFDEF OCAML_VERSION <= OCAML_3_08_4 THEN
-        failwith "Rprint.print_out_sig_item: not implemented case"
-      ELSE
-        match x with
+      match x with
         [ Osig_class vir_flag name params clt _ ->
             fprintf ppf "@[<2>class%s@ %a%s@ :@ %a@]"
               (if vir_flag then " virtual" else "") print_out_class_params
@@ -465,29 +592,31 @@ and print_out_sig_item ppf =
             fprintf ppf "@[<2>class type%s@ %a%s@ =@ %a@]"
               (if vir_flag then " virtual" else "") print_out_class_params
               params name print_out_class_type clt
-        | _ -> assert False ]
-      END ]
+        | _ -> Pp_outcometree.pp_out_sig_item ppf x ]
+    ]
 and print_out_type_decl kwd ppf x =
   let (name, args, ty, priv, constraints) =
-    IFDEF OCAML_VERSION <= OCAML_3_08_4 THEN
-      let (name, args, ty, priv) = x in
-      (name, args, ty, priv, [])
-    ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
-      x
-    ELSE
       (x.otype_name, x.otype_params, x.otype_type, x.otype_private,
        x.otype_cstrs)
-    END
   in
   let constrain ppf (ty, ty') =
     fprintf ppf "@ @[<2>constraint %a =@ %a@]" print_out_type ty
       print_out_type ty'
   in
   let print_constraints ppf params = List.iter (constrain ppf) params in
-  let type_parameter ppf (ty, (co, cn)) =
+  let type_parameter ppf (ty, var_inj) =
     let (q, ty) = try_greek ty in
+    IFDEF OCAML_VERSION < OCAML_4_12_0 THEN
+    let (co, cn) = var_inj in
     fprintf ppf "%s%s%s" (if not cn then "+" else if not co then "-" else "")
       q ty
+    ELSE
+    let (vari, inj) = var_inj in
+    fprintf ppf "%s%s%s%s"
+      (match vari with [ Asttypes.Covariant -> "+" | Contravariant -> "-" | NoVariance -> "" ])
+      (match inj with [ Asttypes.Injective -> "!" | NoInjectivity -> "" ])
+      q ty
+    END
   in
   let type_defined ppf =
     match args with
@@ -537,14 +666,21 @@ value print_out_phrase ppf =
   | Ophr_exception (exn, outv) -> print_out_exception ppf exn outv ]
 ;
 
+value recover_from_print_failure ~{fname} f fallbackf ppf v =
+  try f ppf v
+  with e -> do {
+    Format.fprintf ppf "%!\n================\n%!EXN %s\nPlease report this error\n%!" (Printexc.to_string e) ;
+    fallbackf ppf v
+  }
+;
+
 Toploop.print_out_value.val := print_out_value;
 Toploop.print_out_type.val := print_out_type;
-IFDEF OCAML_VERSION >= OCAML_3_05 THEN
   Toploop.print_out_class_type.val := print_out_class_type;
-  Toploop.print_out_module_type.val := print_out_module_type;
-  Toploop.print_out_signature.val := print_out_signature;
-END;
-Toploop.print_out_sig_item.val := print_out_sig_item;
+  Toploop.print_out_module_type.val :=
+  recover_from_print_failure ~{fname="print_out_module_type"} print_out_module_type Toploop.print_out_module_type.val;
+Toploop.print_out_signature.val :=
+  recover_from_print_failure ~{fname="print_out_signature"} print_out_signature Pp_outcometree.pp_out_sig_item_list;
+  Toploop.print_out_sig_item.val :=
+  recover_from_print_failure ~{fname="print_out_sig_item"} print_out_sig_item Toploop.print_out_sig_item.val;
 Toploop.print_out_phrase.val := print_out_phrase;
-
-END;
